@@ -7,8 +7,8 @@
 
 #define LIBRSVG_MAJOR_VERSION (2)
 #define LIBRSVG_MINOR_VERSION (45)
-#define LIBRSVG_MICRO_VERSION (4)
-#define LIBRSVG_VERSION "2.45.4"
+#define LIBRSVG_MICRO_VERSION (5)
+#define LIBRSVG_VERSION "2.45.5"
 
 #define LIBRSVG_CHECK_VERSION(major,minor,micro) \
   (LIBRSVG_MAJOR_VERSION > (major) || \
@@ -22,6 +22,8 @@
 #define LIBRSVG_CHECK_FEATURE(FEATURE) (defined(LIBRSVG_HAVE_##FEATURE) && LIBRSVG_HAVE_##FEATURE)
 #endif
 
+#ifndef __GTK_DOC_IGNORE__
+
 /*
  * On Windows builds, we need to decorate variables that are exposed in the public API
  * so that they can be properly exported and linked to, for DLL builds
@@ -42,6 +44,9 @@
 # endif /* !G_PLATFORM_WIN32 */
 #endif
 
+#endif /* __GTK_DOC_IGNORE__ */
+
+
 RSVG_VAR const guint librsvg_major_version;
 RSVG_VAR const guint librsvg_minor_version;
 RSVG_VAR const guint librsvg_micro_version;
