@@ -31,9 +31,10 @@
  * or in the most general form, from a #GInputStream that will provide SVG data.
  *
  * Librsvg supports reading <link
- * xlink:href="https://www.w3.org/TR/SVG/">SVG 1.1</link> data.  It also
- * supports SVGZ files, which is just an SVG stream compressed with the GZIP
- * algorithm.
+ * href="https://www.w3.org/TR/SVG11/">SVG 1.1</link> data, and is
+ * gradually adding support for features in <link
+ * href="https://www.w3.org/TR/SVG2/">SVG 2</link>.  Librsvg also supports
+ * SVGZ files, which is just an SVG stream compressed with the GZIP algorithm.
  *
  * # The "base file" and resolving references to external files
  *
@@ -43,7 +44,7 @@
  * and that it has an image element like this:
  *
  * |[
- * <image xlink:href="resources/foo.png" .../>
+ * <image href="resources/foo.png" .../>
  * ]|
  *
  * In this case, librsvg needs to know the location of the toplevel
@@ -61,8 +62,58 @@
  * <filename>/foo/bar/<!-- -->*<!-- -->/.../<!-- -->*</filename>.  This is so that malicious
  * SVG files cannot include files that are in a directory above.
  *
+ * The full set of rules for deciding which URLs may be loaded is as follows;
+ * they are applied in order.  A referenced URL will not be loaded as soon as
+ * one of these rules fails:
+ *
+ * <orderedlist>
+ *   <listitem>
+ *     All <literal>data:</literal> URLs may be loaded.  These are sometimes used
+ *     to include raster image data, encoded as base-64, directly in an SVG file.
+ *   </listitem>
+ *
+ *   <listitem>
+ *     All other URL schemes in references require a base URL.  For
+ *     example, this means that if you load an SVG with
+ *     rsvg_handle_new_from_data() without calling rsvg_handle_set_base_uri(),
+ *     then any referenced files will not be allowed (e.g. raster images to be
+ *     loaded from other files will not work).
+ *   </listitem>
+ *
+ *   <listitem>
+ *     If referenced URLs are absolute, rathen than relative, then they must
+ *     have the same scheme as the base URL.  For example, if the base URL has a
+ *     "<literal>file</literal>" scheme, then all URL references inside the SVG must
+ *     also have the "<literal>file</literal>" scheme, or be relative references which
+ *     will be resolved against the base URL.
+ *   </listitem>
+ *
+ *   <listitem>
+ *     If referenced URLs have a "<literal>resource</literal>" scheme, that is,
+ *     if they are included into your binary program with GLib's resource
+ *     mechanism, they are allowed to be loaded (provided that the base URL is
+ *     also a "<literal>resource</literal>", per the previous rule).
+ *   </listitem>
+ *
+ *   <listitem>
+ *     Otherwise, non-<literal>file</literal> schemes are not allowed.  For
+ *     example, librsvg will not load <literal>http</literal> resources, to keep
+ *     malicious SVG data from "phoning home".
+ *   </listitem>
+ *
+ *   <listitem>
+ *     A relative URL must resolve to the same directory as the base URL, or to
+ *     one of its subdirectories.  Librsvg will canonicalize filenames, by
+ *     removing ".." path components and resolving symbolic links, to decide whether
+ *     files meet these conditions.
+ *   </listitem>
+ * </orderedlist>
+ *
  * # Loading an SVG with GIO
  *
+ * This is the easiest and most resource-efficient way of loading SVG data into
+ * an #RsvgHandle.
+ *
  * If you have a #GFile that stands for an SVG file, you can simply call
  * rsvg_handle_new_from_gfile_sync() to load an RsvgHandle from it.
  *
@@ -74,11 +125,9 @@
  *
  * ## Loading an SVG from memory
  *
- * If you already have SVG data in memory, you can create a memory input stream
- * with g_memory_input_stream_new_from_data() and feed that to
- * rsvg_handle_new_from_stream_sync().  This lets you specify the appropriate
- * flags, for example #RSVG_HANDLE_FLAG_UNLIMITED if your input data is very
- * large.
+ * If you already have SVG data in a byte buffer in memory, you can create a
+ * memory input stream with g_memory_input_stream_new_from_data() and feed that
+ * to rsvg_handle_new_from_stream_sync().
  *
  * Note that in this case, it is important that you specify the base_file for
  * the in-memory SVG data.  Librsvg uses the base_file to resolve links to
@@ -88,13 +137,15 @@
  *
  * You can load an RsvgHandle from a simple filename or URI with
  * rsvg_handle_new_from_file().  Note that this is a blocking operation; there
- * is no way to cancel it if loading a remote URI takes a long time.
+ * is no way to cancel it if loading a remote URI takes a long time.  Also, note that
+ * this method does not let you specify #RsvgHandleFlags.
  *
- * Alternatively, you can create an empty RsvgHandle with rsvg_handle_new() or
- * rsvg_handle_new_with_flags().  The first function is equivalent to using
- * #RSVG_HANDLE_FLAGS_NONE on the second one.  These functions give you back an
- * empty RsvgHandle, which is ready for you to feed it SVG data.  You can do
- * this with rsvg_handle_write() and rsvg_handle_close().
+ * Otherwise, loading an SVG without GIO is not recommended, since librsvg will
+ * need to buffer your entire data internally before actually being able to
+ * parse it.  The deprecated way of doing this is by creating a handle with
+ * rsvg_handle_new() or rsvg_handle_new_with_flags(), and then using
+ * rsvg_handle_write() and rsvg_handle_close() to feed the handle with SVG data.
+ * Still, please try to use the GIO stream functions instead.
  *
  * # Resolution of the rendered image (dots per inch, or DPI)
  *
@@ -117,6 +168,96 @@
  * the dimensions that librsvg is able to compute from the SVG data.
  */
 
+/***** Begin documentation for RsvgHandle properties *****/
+
+/**
+ * RsvgHandle:flags:
+ *
+ * Flags from #RsvgHandleFlags.
+ *
+ * Since: 2.36
+ */
+
+/**
+ * RsvgHandle::dpi-x:
+ *
+ * Horizontal resolution in dots per inch.
+ */
+
+/**
+ * RsvgHandle::dpi-y:
+ *
+ * Horizontal resolution in dots per inch.
+ */
+
+/**
+ * RsvgHandle::base-uri:
+ *
+ * Base URI, to be used to resolve relative references for resources.  See the section
+ */
+
+/**
+ * RsvgHandle:width:
+ *
+ * Width, in pixels, of the rendered SVG after calling the size callback
+ * as specified by rsvg_handle_set_size_callback().
+ *
+ * Deprecated: 2.46. Please use rsvg_handle_get_geometry_sub() instead.
+ */
+
+/**
+ * RsvgHandle:height:
+ *
+ * Height, in pixels, of the rendered SVG after calling the size callback
+ * as specified by rsvg_handle_set_size_callback().
+ *
+ * Deprecated: 2.46. Please use rsvg_handle_get_geometry_sub() instead.
+ */
+
+/**
+ * RsvgHandle:em:
+ *
+ * Exact width, in pixels, of the rendered SVG before calling the size callback
+ * as specified by rsvg_handle_set_size_callback().
+ *
+ * Deprecated: 2.46. Please use rsvg_handle_get_geometry_sub() instead.
+ */
+
+/**
+ * RsvgHandle:ex:
+ *
+ * Exact height, in pixels, of the rendered SVG before calling the size callback
+ * as specified by rsvg_handle_set_size_callback().
+ *
+ * Deprecated: 2.46. Please use rsvg_handle_get_geometry_sub() instead.
+ */
+
+/**
+ * RsvgHandle:title:
+ *
+ * SVG's title.
+ *
+ * Deprecated: 2.36.  Reading this property always returns #NULL.
+ */
+
+/**
+ * RsvgHandle:desc:
+ *
+ * SVG's description.
+ *
+ * Deprecated: 2.36.  Reading this property always returns #NULL.
+ */
+
+/**
+ * RsvgHandle:metadata:
+ *
+ * SVG's metadata
+ *
+ * Deprecated: 2.36.  Reading this property always returns #NULL.
+ */
+
+/***** End documentation for RsvgHandle properties *****/
+
 #include "config.h"
 #define _GNU_SOURCE 1
 
@@ -143,17 +284,17 @@ RsvgHandleRust *rsvg_handle_get_rust (RsvgHandle *handle);
 /* Implemented in rsvg_internals/src/handle.rs */
 extern RsvgHandleRust *rsvg_handle_rust_new (void);
 extern void rsvg_handle_rust_free (RsvgHandleRust *raw_handle);
-extern double rsvg_handle_rust_get_dpi_x (RsvgHandleRust *raw_handle);
-extern double rsvg_handle_rust_get_dpi_y (RsvgHandleRust *raw_handle);
-extern void rsvg_handle_rust_set_dpi_x (RsvgHandleRust *raw_handle, double dpi_x);
-extern void rsvg_handle_rust_set_dpi_y (RsvgHandleRust *raw_handle, double dpi_y);
-extern void rsvg_handle_rust_set_base_url (RsvgHandleRust *raw_handle, const char *uri);
-extern void rsvg_handle_rust_set_base_gfile (RsvgHandleRust *raw_handle, GFile *file);
-extern const char *rsvg_handle_rust_get_base_url (RsvgHandleRust *raw_handle);
-extern GFile *rsvg_handle_rust_get_base_gfile (RsvgHandleRust *raw_handle);
-extern guint rsvg_handle_rust_get_flags (RsvgHandleRust *raw_handle);
-extern void rsvg_handle_rust_set_flags (RsvgHandleRust *raw_handle, guint flags);
-extern guint rsvg_handle_rust_set_testing (RsvgHandleRust *raw_handle, gboolean testing);
+extern double rsvg_handle_rust_get_dpi_x (RsvgHandle *raw_handle);
+extern double rsvg_handle_rust_get_dpi_y (RsvgHandle *raw_handle);
+extern void rsvg_handle_rust_set_dpi_x (RsvgHandle *raw_handle, double dpi_x);
+extern void rsvg_handle_rust_set_dpi_y (RsvgHandle *raw_handle, double dpi_y);
+extern void rsvg_handle_rust_set_base_url (RsvgHandle *raw_handle, const char *uri);
+extern void rsvg_handle_rust_set_base_gfile (RsvgHandle *raw_handle, GFile *file);
+extern const char *rsvg_handle_rust_get_base_url (RsvgHandle *raw_handle);
+extern GFile *rsvg_handle_rust_get_base_gfile (RsvgHandle *raw_handle);
+extern guint rsvg_handle_rust_get_flags (RsvgHandle *raw_handle);
+extern void rsvg_handle_rust_set_flags (RsvgHandle *raw_handle, guint flags);
+extern guint rsvg_handle_rust_set_testing (RsvgHandle *raw_handle, gboolean testing);
 extern gboolean rsvg_handle_rust_read_stream_sync (RsvgHandle *handle,
                                                    GInputStream *stream,
                                                    GCancellable *cancellable,
@@ -177,19 +318,33 @@ extern gboolean rsvg_handle_rust_get_dimensions_sub (RsvgHandle *handle,
 extern gboolean rsvg_handle_rust_get_position_sub (RsvgHandle *handle,
                                                    RsvgPositionData *dimension_data,
                                                    const char *id);
-extern void rsvg_handle_rust_set_size_callback (RsvgHandleRust *raw_handle,
+extern void rsvg_handle_rust_set_size_callback (RsvgHandle *raw_handle,
                                                 RsvgSizeFunc size_func,
                                                 gpointer user_data,
                                                 GDestroyNotify destroy_notify);
+extern RsvgHandle *rsvg_handle_rust_new_with_flags (RsvgHandleFlags flags);
+extern RsvgHandle *rsvg_handle_rust_new_from_file (const char *filename,
+                                                   GError **error);
+extern RsvgHandle *rsvg_handle_rust_new_from_gfile_sync (GFile *file,
+                                                         RsvgHandleFlags flags,
+                                                         GCancellable *cancellable,
+                                                         GError **error);
 extern RsvgHandle *rsvg_handle_rust_new_from_stream_sync (GInputStream *input_stream,
                                                           GFile *base_file,
                                                           RsvgHandleFlags flags,
                                                           GCancellable *cancellable,
                                                           GError **error);
+extern RsvgHandle *rsvg_handle_rust_new_from_data (const guint8 *data,
+                                                   gsize data_len,
+                                                   GError **error);
 
-struct RsvgHandlePrivate {
+/* Implemented in rsvg_internals/src/c_api.rs */
+extern GType rsvg_rust_error_get_type (void);
+extern GType rsvg_rust_handle_flags_get_type (void);
+
+typedef struct {
     RsvgHandleRust *rust_handle;
-};
+} RsvgHandlePrivate;
 
 enum {
     PROP_0,
@@ -211,18 +366,19 @@ G_DEFINE_TYPE_WITH_CODE (RsvgHandle, rsvg_handle, G_TYPE_OBJECT,
                          G_ADD_PRIVATE (RsvgHandle))
 
 static void
-rsvg_handle_init (RsvgHandle * self)
+rsvg_handle_init (RsvgHandle *self)
 {
-    self->priv = rsvg_handle_get_instance_private (self);
-    self->priv->rust_handle = rsvg_handle_rust_new();
+    RsvgHandlePrivate *priv = rsvg_handle_get_instance_private (self);
+    priv->rust_handle = rsvg_handle_rust_new();
 }
 
 static void
 rsvg_handle_dispose (GObject *instance)
 {
     RsvgHandle *self = (RsvgHandle *) instance;
+    RsvgHandlePrivate *priv = rsvg_handle_get_instance_private (self);
 
-    g_clear_pointer (&self->priv->rust_handle, rsvg_handle_rust_free);
+    g_clear_pointer (&priv->rust_handle, rsvg_handle_rust_free);
 
     G_OBJECT_CLASS (rsvg_handle_parent_class)->dispose (instance);
 }
@@ -234,13 +390,13 @@ rsvg_handle_set_property (GObject * instance, guint prop_id, GValue const *value
 
     switch (prop_id) {
     case PROP_FLAGS:
-        rsvg_handle_rust_set_flags (self->priv->rust_handle, g_value_get_flags (value));
+        rsvg_handle_rust_set_flags (self, g_value_get_flags (value));
         break;
     case PROP_DPI_X:
-        rsvg_handle_rust_set_dpi_x (self->priv->rust_handle, g_value_get_double (value));
+        rsvg_handle_rust_set_dpi_x (self, g_value_get_double (value));
         break;
     case PROP_DPI_Y:
-        rsvg_handle_rust_set_dpi_y (self->priv->rust_handle, g_value_get_double (value));
+        rsvg_handle_rust_set_dpi_y (self, g_value_get_double (value));
         break;
     case PROP_BASE_URI: {
         const char *str = g_value_get_string (value);
@@ -264,13 +420,13 @@ rsvg_handle_get_property (GObject * instance, guint prop_id, GValue * value, GPa
 
     switch (prop_id) {
     case PROP_FLAGS:
-        g_value_set_flags (value, rsvg_handle_rust_get_flags (self->priv->rust_handle));
+        g_value_set_flags (value, rsvg_handle_rust_get_flags (self));
         break;
     case PROP_DPI_X:
-        g_value_set_double (value, rsvg_handle_rust_get_dpi_x (self->priv->rust_handle));
+        g_value_set_double (value, rsvg_handle_rust_get_dpi_x (self));
         break;
     case PROP_DPI_Y:
-        g_value_set_double (value, rsvg_handle_rust_get_dpi_y (self->priv->rust_handle));
+        g_value_set_double (value, rsvg_handle_rust_get_dpi_y (self));
         break;
     case PROP_BASE_URI:
         g_value_set_string (value, rsvg_handle_get_base_uri (self));
@@ -426,7 +582,7 @@ rsvg_handle_class_init (RsvgHandleClass * klass)
  * Deprecated: Use g_object_unref() instead.
  **/
 void
-rsvg_handle_free (RsvgHandle * handle)
+rsvg_handle_free (RsvgHandle *handle)
 {
     g_object_unref (handle);
 }
@@ -441,13 +597,20 @@ rsvg_handle_free (RsvgHandle * handle)
  * rsvg_handle_read_stream_sync().
  *
  * The deprecated way of loading SVG data is with rsvg_handle_write() and
- * rsvg_handle_close().
+ * rsvg_handle_close(); note that these require buffering the entire file
+ * internally, and for this reason it is better to use the stream functions:
+ * rsvg_handle_new_from_stream_sync(), rsvg_handle_read_stream_sync(), or
+ * rsvg_handle_new_from_gfile_sync().
  *
- * After loading the #RsvgHandl with data, you can render it using Cairo or get
- * a GdkPixbuf from it. When finished, free with g_object_unref(). No more than
- * one image can be loaded with one handle.
+ * After loading the #RsvgHandle with data, you can render it using Cairo or get
+ * a GdkPixbuf from it. When finished, free the handle with g_object_unref(). No
+ * more than one image can be loaded with one handle.
  *
- * Returns: A new #RsvgHandle
+ * Note that this function creates an #RsvgHandle with no flags set.  If you require
+ * any of #RsvgHandleFlags to be set, use rsvg_handle_new_with_flags() or the stream
+ * functions listed above.
+ *
+ * Returns: A new #RsvgHandle with no flags set.
  **/
 RsvgHandle *
 rsvg_handle_new (void)
@@ -462,7 +625,7 @@ rsvg_handle_new (void)
  * @error: return location for errors
  *
  * Loads the SVG specified by @data.  Note that this function creates an
- * #RsvgHandle without a base file, and without any special flags.  If you
+ * #RsvgHandle without a base URL, and without any #RsvgHandleFlags.  If you
  * need these, use rsvg_handle_new_from_stream_sync() instead by creating
  * a #GMemoryInputStream from your data.
  *
@@ -472,23 +635,16 @@ rsvg_handle_new (void)
 RsvgHandle *
 rsvg_handle_new_from_data (const guint8 *data, gsize data_len, GError **error)
 {
-    GInputStream *stream;
-    RsvgHandle *handle;
-
-    g_return_val_if_fail (data != NULL, NULL);
+    g_return_val_if_fail ((data != NULL && data_len != 0) || (data_len == 0), NULL);
     g_return_val_if_fail (data_len <= G_MAXSSIZE, NULL);
     g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
-    stream = g_memory_input_stream_new_from_data (data, data_len, NULL);
-    handle = rsvg_handle_new_from_stream_sync (stream, NULL, RSVG_HANDLE_FLAGS_NONE, NULL, error);
-    g_object_unref (stream);
-
-    return handle;
+    return rsvg_handle_rust_new_from_data (data, data_len, error);
 }
 
 /**
  * rsvg_handle_new_from_file:
- * @file_name: The file name to load. If built with gnome-vfs, can be a URI.
+ * @filename: The file name to load, or a URI.
  * @error: return location for errors
  *
  * Loads the SVG specified by @file_name.
@@ -497,39 +653,12 @@ rsvg_handle_new_from_data (const guint8 *data, gsize data_len, GError **error)
  * Since: 2.14
  */
 RsvgHandle *
-rsvg_handle_new_from_file (const gchar *file_name, GError **error)
+rsvg_handle_new_from_file (const gchar *filename, GError **error)
 {
-    gchar *base_uri;
-    RsvgHandle *handle;
-    GFile *file;
-    char *scheme;
-
-    g_return_val_if_fail (file_name != NULL, NULL);
+    g_return_val_if_fail (filename != NULL, NULL);
     g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
-    scheme = g_uri_parse_scheme (file_name);
-    if (scheme) {
-        file = g_file_new_for_uri (file_name);
-        g_free (scheme);
-    } else {
-        file = g_file_new_for_path (file_name);
-    }
-
-    base_uri = g_file_get_uri (file);
-    if (!base_uri) {
-        g_set_error (error,
-                     G_IO_ERROR,
-                     G_IO_ERROR_FAILED,
-                     _("Cannot obtain URI from '%s'"), file_name);
-        g_object_unref (file);
-        return NULL;
-    }
-    g_free (base_uri);
-
-    handle = rsvg_handle_new_from_gfile_sync (file, 0, NULL, error);
-    g_object_unref (file);
-
-    return handle;
+    return rsvg_handle_rust_new_from_file (filename, error);
 }
 
 /**
@@ -545,9 +674,7 @@ rsvg_handle_new_from_file (const gchar *file_name, GError **error)
 RsvgHandle *
 rsvg_handle_new_with_flags (RsvgHandleFlags flags)
 {
-    return g_object_new (RSVG_TYPE_HANDLE,
-                         "flags", flags,
-                         NULL);
+    return rsvg_handle_rust_new_with_flags (flags);
 }
 
 /**
@@ -574,22 +701,11 @@ rsvg_handle_new_from_gfile_sync (GFile          *file,
                                  GCancellable   *cancellable,
                                  GError        **error)
 {
-    RsvgHandle *handle;
-    GFileInputStream *stream;
-
     g_return_val_if_fail (G_IS_FILE (file), NULL);
     g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
     g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
-    stream = g_file_read (file, cancellable, error);
-    if (stream == NULL)
-        return NULL;
-
-    handle = rsvg_handle_new_from_stream_sync (G_INPUT_STREAM (stream), file,
-                                               flags, cancellable, error);
-    g_object_unref (stream);
-
-    return handle;
+    return rsvg_handle_rust_new_from_gfile_sync (file, flags, cancellable, error);
 }
 
 /**
@@ -612,9 +728,9 @@ rsvg_handle_new_from_gfile_sync (GFile          *file,
  * Since: 2.32
  */
 RsvgHandle *
-rsvg_handle_new_from_stream_sync (GInputStream   *input_stream,
-                                  GFile          *base_file,
-                                  RsvgHandleFlags flags,
+rsvg_handle_new_from_stream_sync (GInputStream    *input_stream,
+                                  GFile           *base_file,
+                                  RsvgHandleFlags  flags,
                                   GCancellable    *cancellable,
                                   GError         **error)
 {
@@ -729,12 +845,12 @@ rsvg_handle_read_stream_sync (RsvgHandle   *handle,
  * Since: 2.9
  */
 void
-rsvg_handle_set_base_uri (RsvgHandle * handle, const char *base_uri)
+rsvg_handle_set_base_uri (RsvgHandle *handle, const char *base_uri)
 {
     g_return_if_fail (RSVG_IS_HANDLE (handle));
     g_return_if_fail (base_uri != NULL);
 
-    rsvg_handle_rust_set_base_url (handle->priv->rust_handle, base_uri);
+    rsvg_handle_rust_set_base_url (handle, base_uri);
 }
 
 /**
@@ -755,7 +871,7 @@ rsvg_handle_set_base_gfile (RsvgHandle *handle,
     g_return_if_fail (RSVG_IS_HANDLE (handle));
     g_return_if_fail (G_IS_FILE (base_file));
 
-    rsvg_handle_rust_set_base_gfile (handle->priv->rust_handle, base_file);
+    rsvg_handle_rust_set_base_gfile (handle, base_file);
 }
 
 /**
@@ -768,11 +884,11 @@ rsvg_handle_set_base_gfile (RsvgHandle *handle,
  * Since: 2.8
  */
 const char *
-rsvg_handle_get_base_uri (RsvgHandle * handle)
+rsvg_handle_get_base_uri (RsvgHandle *handle)
 {
-    g_return_val_if_fail (handle, NULL);
+    g_return_val_if_fail (RSVG_IS_HANDLE (handle), NULL);
 
-    return rsvg_handle_rust_get_base_url (handle->priv->rust_handle);
+    return rsvg_handle_rust_get_base_url (handle);
 }
 
 /**
@@ -806,7 +922,7 @@ rsvg_handle_get_metadata (RsvgHandle * handle)
  * this function always returns #NULL.
  */
 const char *
-rsvg_handle_get_title (RsvgHandle * handle)
+rsvg_handle_get_title (RsvgHandle *handle)
 {
     g_return_val_if_fail (handle, NULL);
 
@@ -825,7 +941,7 @@ rsvg_handle_get_title (RsvgHandle * handle)
  * this function always returns #NULL.
  */
 const char *
-rsvg_handle_get_desc (RsvgHandle * handle)
+rsvg_handle_get_desc (RsvgHandle *handle)
 {
     g_return_val_if_fail (handle, NULL);
 
@@ -835,7 +951,8 @@ rsvg_handle_get_desc (RsvgHandle * handle)
 RsvgHandleRust *
 rsvg_handle_get_rust (RsvgHandle *handle)
 {
-    return handle->priv->rust_handle;
+    RsvgHandlePrivate *priv = rsvg_handle_get_instance_private (handle);
+    return priv->rust_handle;
 }
 
 /**
@@ -855,7 +972,7 @@ rsvg_handle_get_rust (RsvgHandle *handle)
  * Since: 2.14
  */
 gboolean
-rsvg_handle_render_cairo_sub (RsvgHandle * handle, cairo_t * cr, const char *id)
+rsvg_handle_render_cairo_sub (RsvgHandle *handle, cairo_t *cr, const char *id)
 {
     g_return_val_if_fail (RSVG_IS_HANDLE (handle), FALSE);
     g_return_val_if_fail (cr != NULL, FALSE);
@@ -877,7 +994,7 @@ rsvg_handle_render_cairo_sub (RsvgHandle * handle, cairo_t * cr, const char *id)
  * Since: 2.14
  */
 gboolean
-rsvg_handle_render_cairo (RsvgHandle * handle, cairo_t * cr)
+rsvg_handle_render_cairo (RsvgHandle *handle, cairo_t *cr)
 {
     return rsvg_handle_render_cairo_sub (handle, cr, NULL);
 }
@@ -892,7 +1009,7 @@ rsvg_handle_render_cairo (RsvgHandle * handle, cairo_t * cr)
  * Since: 2.14
  */
 void
-rsvg_handle_get_dimensions (RsvgHandle * handle, RsvgDimensionData * dimension_data)
+rsvg_handle_get_dimensions (RsvgHandle *handle, RsvgDimensionData *dimension_data)
 {
     g_return_if_fail (RSVG_IS_HANDLE (handle));
     g_return_if_fail (dimension_data != NULL);
@@ -915,7 +1032,9 @@ rsvg_handle_get_dimensions (RsvgHandle * handle, RsvgDimensionData * dimension_d
  * Since: 2.22
  */
 gboolean
-rsvg_handle_get_dimensions_sub (RsvgHandle * handle, RsvgDimensionData * dimension_data, const char *id)
+rsvg_handle_get_dimensions_sub (RsvgHandle *handle,
+                                RsvgDimensionData *dimension_data,
+                                const char *id)
 {
     g_return_val_if_fail (RSVG_IS_HANDLE (handle), FALSE);
     g_return_val_if_fail (dimension_data, FALSE);
@@ -926,8 +1045,8 @@ rsvg_handle_get_dimensions_sub (RsvgHandle * handle, RsvgDimensionData * dimensi
 /**
  * rsvg_handle_get_geometry_sub:
  * @handle: A #RsvgHandle
- * @ink_rect: (out)(nullable): A place to store the SVG fragment's geometry.
- * @logical_rect: (out)(nullable): A place to store the SVG fragment's logical geometry.
+ * @ink_rect: (out)(allow-none): A place to store the SVG fragment's geometry.
+ * @logical_rect: (out)(allow-none): A place to store the SVG fragment's logical geometry.
  * @id: (nullable): An element's id within the SVG, starting with "##", for
  * example, "##layer1"; or %NULL to use the whole SVG.
  *
@@ -939,7 +1058,10 @@ rsvg_handle_get_dimensions_sub (RsvgHandle * handle, RsvgDimensionData * dimensi
  * Since: 2.46
  */
 gboolean
-rsvg_handle_get_geometry_sub (RsvgHandle * handle, RsvgRectangle * ink_rect, RsvgRectangle * logical_rect, const char *id)
+rsvg_handle_get_geometry_sub (RsvgHandle *handle,
+                              RsvgRectangle *ink_rect,
+                              RsvgRectangle *logical_rect,
+                              const char *id)
 {
     g_return_val_if_fail (RSVG_IS_HANDLE (handle), FALSE);
 
@@ -961,7 +1083,9 @@ rsvg_handle_get_geometry_sub (RsvgHandle * handle, RsvgRectangle * ink_rect, Rsv
  * Since: 2.22
  */
 gboolean
-rsvg_handle_get_position_sub (RsvgHandle * handle, RsvgPositionData * position_data, const char *id)
+rsvg_handle_get_position_sub (RsvgHandle *handle,
+                              RsvgPositionData *position_data,
+                              const char *id)
 {
     g_return_val_if_fail (RSVG_IS_HANDLE (handle), FALSE);
     g_return_val_if_fail (position_data != NULL, FALSE);
@@ -981,8 +1105,7 @@ rsvg_handle_get_position_sub (RsvgHandle * handle, RsvgPositionData * position_d
  * Since: 2.22
  */
 gboolean
-rsvg_handle_has_sub (RsvgHandle *handle,
-                     const char *id)
+rsvg_handle_has_sub (RsvgHandle *handle, const char *id)
 {
     g_return_val_if_fail (RSVG_IS_HANDLE (handle), FALSE);
 
@@ -1012,7 +1135,7 @@ rsvg_handle_has_sub (RsvgHandle *handle,
  * Since: 2.14
  **/
 GdkPixbuf *
-rsvg_handle_get_pixbuf_sub (RsvgHandle * handle, const char *id)
+rsvg_handle_get_pixbuf_sub (RsvgHandle *handle, const char *id)
 {
     g_return_val_if_fail (RSVG_IS_HANDLE (handle), NULL);
 
@@ -1032,7 +1155,7 @@ rsvg_handle_get_pixbuf_sub (RsvgHandle * handle, const char *id)
  * Returns: (transfer full) (nullable): the pixbuf loaded by @handle, or %NULL.
  **/
 GdkPixbuf *
-rsvg_handle_get_pixbuf (RsvgHandle * handle)
+rsvg_handle_get_pixbuf (RsvgHandle *handle)
 {
     return rsvg_handle_get_pixbuf_sub (handle, NULL);
 }
@@ -1049,7 +1172,7 @@ rsvg_handle_get_pixbuf (RsvgHandle * handle)
  * Since: 2.8
  */
 void
-rsvg_handle_set_dpi (RsvgHandle * handle, double dpi)
+rsvg_handle_set_dpi (RsvgHandle *handle, double dpi)
 {
     rsvg_handle_set_dpi_x_y (handle, dpi, dpi);
 }
@@ -1067,12 +1190,12 @@ rsvg_handle_set_dpi (RsvgHandle * handle, double dpi)
  * Since: 2.8
  */
 void
-rsvg_handle_set_dpi_x_y (RsvgHandle * handle, double dpi_x, double dpi_y)
+rsvg_handle_set_dpi_x_y (RsvgHandle *handle, double dpi_x, double dpi_y)
 {
     g_return_if_fail (RSVG_IS_HANDLE (handle));
 
-    rsvg_handle_rust_set_dpi_x (handle->priv->rust_handle, dpi_x);
-    rsvg_handle_rust_set_dpi_y (handle->priv->rust_handle, dpi_y);
+    rsvg_handle_rust_set_dpi_x (handle, dpi_x);
+    rsvg_handle_rust_set_dpi_y (handle, dpi_y);
 }
 
 /**
@@ -1080,15 +1203,26 @@ rsvg_handle_set_dpi_x_y (RsvgHandle * handle, double dpi_x, double dpi_y)
  * @handle: An #RsvgHandle
  * @size_func: (nullable): A sizing function, or %NULL
  * @user_data: User data to pass to @size_func, or %NULL
- * @user_data_destroy: Destroy function for @user_data, or %NULL
+ * @user_data_destroy: Function to be called to destroy the data passed in @user_data,
+ *   or %NULL.
  *
- * Sets the sizing function for the @handle.  This function is called right
- * after the size of the image has been loaded.  The size of the image is passed
- * in to the function, which may then modify these values to set the real size
- * of the generated pixbuf.  If the image has no associated size, then the size
- * arguments are set to -1.
+ * Sets the sizing function for the @handle, which can be used to override the
+ * size that librsvg computes for SVG images.  The @size_func is called from the
+ * following functions:
  *
- * Deprecated: Set up a cairo matrix and use rsvg_handle_render_cairo() instead.
+ * <itemizedlist>
+ *   <listitem>rsvg_handle_get_dimensions()</listitem>
+ *   <listitem>rsvg_handle_get_dimensions_sub()</listitem>
+ *   <listitem>rsvg_handle_get_position_sub()</listitem>
+ *   <listitem>rsvg_handle_render_cairo()</listitem>
+ *   <listitem>rsvg_handle_render_cairo_sub()</listitem>
+ * </itemizedlist>
+ *
+ * Librsvg computes the size of the SVG being rendered, and passes it to the
+ * @size_func, which may then modify these values to set the final size of the
+ * generated image.
+ *
+ * Deprecated: 2.14.  Set up a cairo matrix and use rsvg_handle_render_cairo() instead.
  * You can call rsvg_handle_get_dimensions() to figure out the size of your SVG,
  * and then scale it to the desired size via Cairo.  For example, the following
  * code renders an SVG at a specified size, scaled proportionally from whatever
@@ -1114,16 +1248,22 @@ rsvg_handle_set_dpi_x_y (RsvgHandle * handle, double dpi_x, double dpi_y)
  *     rsvg_handle_render_cairo (handle, cr);
  * }
  * ]|
+ *
+ * This function was deprecated because when the @size_func is used, it makes it
+ * unclear when the librsvg functions which call the @size_func will use the
+ * size computed originally, or the callback-specified size, or whether it
+ * refers to the whole SVG or to just a sub-element of it.  It is easier, and
+ * unambiguous, to use code similar to the example above.
  **/
 void
-rsvg_handle_set_size_callback (RsvgHandle * handle,
+rsvg_handle_set_size_callback (RsvgHandle *handle,
                                RsvgSizeFunc size_func,
                                gpointer user_data,
                                GDestroyNotify user_data_destroy)
 {
     g_return_if_fail (RSVG_IS_HANDLE (handle));
 
-    rsvg_handle_rust_set_size_callback (handle->priv->rust_handle,
+    rsvg_handle_rust_set_size_callback (handle,
                                         size_func,
                                         user_data,
                                         user_data_destroy);
@@ -1142,7 +1282,7 @@ rsvg_handle_internal_set_testing (RsvgHandle *handle, gboolean testing)
 {
     g_return_if_fail (RSVG_IS_HANDLE (handle));
 
-    rsvg_handle_rust_set_testing (handle->priv->rust_handle, testing);
+    rsvg_handle_rust_set_testing (handle, testing);
 }
 
 /* This one is defined in the C code, because the prototype has varargs
@@ -1165,3 +1305,15 @@ rsvg_sax_error_cb (void *data, const char *msg, ...)
 
     g_free (buf);
 }
+
+GType
+rsvg_error_get_type(void)
+{
+    return rsvg_rust_error_get_type();
+}
+
+GType
+rsvg_handle_flags_get_type(void)
+{
+    return rsvg_rust_handle_flags_get_type();
+}
