@@ -36,6 +36,8 @@
 
 G_BEGIN_DECLS
 
+#ifndef __GTK_DOC_IGNORE__
+
 #if defined(RSVG_DISABLE_DEPRECATION_WARNINGS) || !GLIB_CHECK_VERSION (2, 31, 0)
 #define RSVG_DEPRECATED
 #define RSVG_DEPRECATED_FOR(f)
@@ -44,6 +46,8 @@ G_BEGIN_DECLS
 #define RSVG_DEPRECATED_FOR(f) G_DEPRECATED_FOR(f)
 #endif
 
+#endif /* __GTK_DOC_IGNORE__ */
+
 #define RSVG_TYPE_HANDLE                  (rsvg_handle_get_type ())
 #define RSVG_HANDLE(obj)                  (G_TYPE_CHECK_INSTANCE_CAST ((obj), RSVG_TYPE_HANDLE, RsvgHandle))
 #define RSVG_HANDLE_CLASS(klass)          (G_TYPE_CHECK_CLASS_CAST ((klass), RSVG_TYPE_HANDLE, RsvgHandleClass))
@@ -66,8 +70,10 @@ typedef enum {
 #define RSVG_ERROR (rsvg_error_quark ())
 GQuark rsvg_error_quark (void) G_GNUC_CONST;
 
+GType rsvg_error_get_type (void);
+#define RSVG_TYPE_ERROR (rsvg_error_get_type())
+
 typedef struct _RsvgHandle RsvgHandle;
-typedef struct RsvgHandlePrivate RsvgHandlePrivate;
 typedef struct _RsvgHandleClass RsvgHandleClass;
 typedef struct _RsvgDimensionData RsvgDimensionData;
 typedef struct _RsvgPositionData RsvgPositionData;
@@ -97,17 +103,19 @@ struct _RsvgHandle {
 
     /*< private >*/
 
-    RsvgHandlePrivate *priv;
-
-    gpointer _abi_padding[15];
+    gpointer _abi_padding[16];
 };
 
 /**
  * RsvgDimensionData:
  * @width: SVG's width, in pixels
  * @height: SVG's height, in pixels
- * @em: em
- * @ex: ex
+ * @em: SVG's original width, unmodified by #RsvgSizeFunc
+ * @ex: SVG's original height, unmodified by #RsvgSizeFunc
+ *
+ * Dimensions of an SVG image from rsvg_handle_get_dimensions(), or an
+ * individual element from rsvg_handle_get_dimensions_sub().  Please see
+ * the deprecation documentation for those functions.
  *
  * Deprecated: 2.46.  Please use rsvg_handle_get_geometry_sub().
  */
@@ -123,7 +131,8 @@ struct _RsvgDimensionData {
  * @x: position on the x axis
  * @y: position on the y axis
  *
- * Position of an SVG fragment.
+ * Position of an SVG fragment from rsvg_handle_get_position_sub().  Please
+ * the deprecation documentation for that function.
  *
  * Deprecated: 2.46.  Please use rsvg_handle_get_geometry_sub().
  */
@@ -160,46 +169,65 @@ void rsvg_set_default_dpi	(double dpi);
 RSVG_DEPRECATED
 void rsvg_set_default_dpi_x_y	(double dpi_x, double dpi_y);
 
-void rsvg_handle_set_dpi	(RsvgHandle * handle, double dpi);
-void rsvg_handle_set_dpi_x_y	(RsvgHandle * handle, double dpi_x, double dpi_y);
+void rsvg_handle_set_dpi	(RsvgHandle *handle, double dpi);
+void rsvg_handle_set_dpi_x_y	(RsvgHandle *handle, double dpi_x, double dpi_y);
 
 RsvgHandle  *rsvg_handle_new		(void);
 
 RSVG_DEPRECATED_FOR(rsvg_handle_read_stream_sync)
-gboolean     rsvg_handle_write		(RsvgHandle * handle, const guchar * buf, 
-                                         gsize count, GError ** error);
+gboolean     rsvg_handle_write		(RsvgHandle   *handle,
+                                         const guchar *buf, 
+                                         gsize         count,
+                                         GError      **error);
 RSVG_DEPRECATED_FOR(rsvg_handle_read_stream_sync)
-gboolean     rsvg_handle_close		(RsvgHandle * handle, GError ** error);
+gboolean     rsvg_handle_close		(RsvgHandle *handle, GError **error);
 
-GdkPixbuf   *rsvg_handle_get_pixbuf	(RsvgHandle * handle);
-GdkPixbuf   *rsvg_handle_get_pixbuf_sub (RsvgHandle * handle, const char *id);
+GdkPixbuf   *rsvg_handle_get_pixbuf	(RsvgHandle *handle);
+GdkPixbuf   *rsvg_handle_get_pixbuf_sub (RsvgHandle *handle, const char *id);
 
-const char  *rsvg_handle_get_base_uri (RsvgHandle * handle);
-void         rsvg_handle_set_base_uri (RsvgHandle * handle, const char *base_uri);
+const char  *rsvg_handle_get_base_uri (RsvgHandle *handle);
+void         rsvg_handle_set_base_uri (RsvgHandle *handle, const char *base_uri);
 
-void rsvg_handle_get_dimensions (RsvgHandle * handle, RsvgDimensionData * dimension_data);
+void rsvg_handle_get_dimensions (RsvgHandle *handle, RsvgDimensionData *dimension_data);
 
 RSVG_DEPRECATED_FOR(rsvg_handle_get_geometry_sub)
-gboolean rsvg_handle_get_dimensions_sub (RsvgHandle * handle, RsvgDimensionData * dimension_data, const char *id);
+gboolean rsvg_handle_get_dimensions_sub (RsvgHandle        *handle,
+                                         RsvgDimensionData *dimension_data,
+                                         const char        *id);
 
 RSVG_DEPRECATED_FOR(rsvg_handle_get_geometry_sub)
-gboolean rsvg_handle_get_position_sub (RsvgHandle * handle, RsvgPositionData * position_data, const char *id);
+gboolean rsvg_handle_get_position_sub (RsvgHandle       *handle,
+                                       RsvgPositionData *position_data,
+                                       const char       *id);
 
-gboolean rsvg_handle_get_geometry_sub (RsvgHandle * handle, RsvgRectangle * ink_rect, RsvgRectangle * logical_rect, const char *id);
+gboolean rsvg_handle_get_geometry_sub (RsvgHandle    *handle,
+                                       RsvgRectangle *ink_rect,
+                                       RsvgRectangle *logical_rect,
+                                       const char    *id);
 
-gboolean rsvg_handle_has_sub (RsvgHandle * handle, const char *id);
+gboolean rsvg_handle_has_sub (RsvgHandle *handle, const char *id);
 
 /* GIO APIs */
 
 /**
  * RsvgHandleFlags:
- * @RSVG_HANDLE_FLAGS_NONE: none
- * @RSVG_HANDLE_FLAG_UNLIMITED: Allow any SVG XML without size limitations.
- *   For security reasons, this should only be used for trusted input!
+ * @RSVG_HANDLE_FLAGS_NONE: No flags are set.
+ * @RSVG_HANDLE_FLAG_UNLIMITED: Disable safety limits in the XML parser.
+ *   Libxml2 has <link
+ *   href="https://gitlab.gnome.org/GNOME/libxml2/blob/master/include/libxml/parserInternals.h">several
+ *   limits</link> designed to keep malicious XML content from consuming too
+ *   much memory while parsing.  For security reasons, this should only be used
+ *   for trusted input!
  *   Since: 2.40.3
- * @RSVG_HANDLE_FLAG_KEEP_IMAGE_DATA: Keeps the image data when loading images,
- *  for use by cairo when painting to e.g. a PDF surface. This will make the
- *  resulting PDF file smaller and faster.
+ * @RSVG_HANDLE_FLAG_KEEP_IMAGE_DATA: Use this if the Cairo surface to which you
+ *  are rendering is a PDF, PostScript, SVG, or Win32 Printing surface.  This
+ *  will make librsvg and Cairo use the original, compressed data for images in
+ *  the final output, instead of passing uncompressed images.  This will make a
+ *  Keeps the image data when loading images, for use by cairo when painting to
+ *  e.g. a PDF surface.  For example, this will make the a resulting PDF file
+ *  smaller and faster.  Please see <link
+ *  href="https://www.cairographics.org/manual/cairo-cairo-surface-t.html#cairo-surface-set-mime-data">the
+ *  Cairo documentation</link> for details.
  *  Since: 2.40.3
  */
 typedef enum /*< flags >*/ 
@@ -209,6 +237,9 @@ typedef enum /*< flags >*/
     RSVG_HANDLE_FLAG_KEEP_IMAGE_DATA = 1 << 1
 } RsvgHandleFlags;
 
+GType rsvg_handle_flags_get_type (void);
+#define RSVG_TYPE_HANDLE_FLAGS (rsvg_handle_flags_get_type())
+
 RsvgHandle *rsvg_handle_new_with_flags (RsvgHandleFlags flags);
 
 void        rsvg_handle_set_base_gfile (RsvgHandle *handle,
@@ -230,10 +261,12 @@ RsvgHandle *rsvg_handle_new_from_stream_sync (GInputStream   *input_stream,
                                               GCancellable   *cancellable,
                                               GError        **error);
 
-RsvgHandle *rsvg_handle_new_from_data (const guint8 * data, gsize data_len, GError ** error);
-RsvgHandle *rsvg_handle_new_from_file (const gchar * file_name, GError ** error);
+RsvgHandle *rsvg_handle_new_from_data (const guint8 *data, gsize data_len, GError **error);
+RsvgHandle *rsvg_handle_new_from_file (const gchar *filename, GError **error);
 
+#ifndef __GTK_DOC_IGNORE__
 void rsvg_handle_internal_set_testing (RsvgHandle *handle, gboolean testing);
+#endif /* __GTK_DOC_IGNORE__ */
 
 /* BEGIN deprecated APIs. Do not use! */
 
@@ -245,7 +278,7 @@ RSVG_DEPRECATED
 void rsvg_term (void);
 
 RSVG_DEPRECATED_FOR(g_object_unref)
-void rsvg_handle_free (RsvgHandle * handle);
+void rsvg_handle_free (RsvgHandle *handle);
 
 /**
  * RsvgSizeFunc:
@@ -255,41 +288,52 @@ void rsvg_handle_free (RsvgHandle * handle);
  *
  * Function to let a user of the library specify the SVG's dimensions
  *
- * Deprecated: Set up a cairo matrix and use rsvg_handle_render_cairo() instead.
- * See the documentation for rsvg_handle_set_size_callback() for an example.
+ * Deprecated: 2.14.  Set up a cairo matrix and use rsvg_handle_render_cairo() instead.
+ * See the documentation for rsvg_handle_set_size_callback() for an example, and
+ * for the reasons for deprecation.
  */
-/* RSVG_DEPRECATED */ typedef void (*RsvgSizeFunc) (gint * width, gint * height, gpointer user_data);
+typedef void (*RsvgSizeFunc) (gint * width, gint * height, gpointer user_data);
 
 RSVG_DEPRECATED
-void rsvg_handle_set_size_callback (RsvgHandle * handle,
-                                    RsvgSizeFunc size_func,
-                                    gpointer user_data, GDestroyNotify user_data_destroy);
+void rsvg_handle_set_size_callback (RsvgHandle    *handle,
+                                    RsvgSizeFunc   size_func,
+                                    gpointer       user_data,
+                                    GDestroyNotify user_data_destroy);
 
 /* GdkPixbuf convenience API */
 
 RSVG_DEPRECATED
-GdkPixbuf *rsvg_pixbuf_from_file            (const gchar * file_name, GError ** error);
+GdkPixbuf *rsvg_pixbuf_from_file            (const gchar *filename,
+                                             GError     **error);
 RSVG_DEPRECATED
-GdkPixbuf *rsvg_pixbuf_from_file_at_zoom    (const gchar * file_name,
-                                             double x_zoom, double y_zoom, GError ** error);
+GdkPixbuf *rsvg_pixbuf_from_file_at_zoom    (const gchar *filename,
+                                             double       x_zoom,
+                                             double       y_zoom,
+                                             GError     **error);
 RSVG_DEPRECATED
-GdkPixbuf *rsvg_pixbuf_from_file_at_size    (const gchar * file_name, gint width, gint height,
-                                             GError ** error);
+GdkPixbuf *rsvg_pixbuf_from_file_at_size    (const gchar *filename,
+                                             gint         width,
+                                             gint         height,
+                                             GError     **error);
 RSVG_DEPRECATED
-GdkPixbuf *rsvg_pixbuf_from_file_at_max_size    (const gchar * file_name,
-                                                 gint max_width, gint max_height, GError ** error);
+GdkPixbuf *rsvg_pixbuf_from_file_at_max_size    (const gchar *filename,
+                                                 gint         max_width,
+                                                 gint         max_height,
+                                                 GError     **error);
 RSVG_DEPRECATED
-GdkPixbuf *rsvg_pixbuf_from_file_at_zoom_with_max (const gchar * file_name,
-                                                   double x_zoom,
-                                                   double y_zoom,
-                                                   gint max_width, gint max_height, GError ** error);
+GdkPixbuf *rsvg_pixbuf_from_file_at_zoom_with_max (const gchar *filename,
+                                                   double       x_zoom,
+                                                   double       y_zoom,
+                                                   gint         max_width,
+                                                   gint         max_height,
+                                                   GError     **error);
 
 RSVG_DEPRECATED
-const char *rsvg_handle_get_title       (RsvgHandle * handle);
+const char *rsvg_handle_get_title       (RsvgHandle *handle);
 RSVG_DEPRECATED
-const char *rsvg_handle_get_desc        (RsvgHandle * handle);
+const char *rsvg_handle_get_desc        (RsvgHandle *handle);
 RSVG_DEPRECATED
-const char *rsvg_handle_get_metadata    (RsvgHandle * handle);
+const char *rsvg_handle_get_metadata    (RsvgHandle *handle);
 
 #endif /* !__GI_SCANNER__ */
 
@@ -297,7 +341,6 @@ const char *rsvg_handle_get_metadata    (RsvgHandle * handle);
 
 G_END_DECLS
 
-#include "librsvg-enum-types.h"
 #include "librsvg-features.h"
 #include "rsvg-cairo.h"
 
