@@ -1,9 +1,12 @@
 use std::error::{self, Error};
 use std::fmt;
 use std::io;
+use std::ops::Deref;
 use std::path::{Path, PathBuf};
 use url::{self, Url};
 
+use error::HrefError;
+
 /// Wrapper for URLs which are allowed to be loaded
 ///
 /// SVG files can reference other files (PNG/JPEG images, other SVGs,
@@ -104,8 +107,12 @@ impl AllowedUrl {
             Err(AllowedUrlError::NotSiblingOrChildOfBaseFile)
         }
     }
+}
 
-    pub fn url(&self) -> &Url {
+impl Deref for AllowedUrl {
+    type Target = Url;
+
+    fn deref(&self) -> &Url {
         &self.0
     }
 }
@@ -151,6 +158,86 @@ fn canonicalize<P: AsRef<Path>>(path: P) -> Result<PathBuf, io::Error> {
     Ok(path.as_ref().to_path_buf())
 }
 
+/// Parsed result of an href from an SVG or CSS file
+///
+/// Sometimes in SVG element references (e.g. the `href` in the `<feImage>` element) we
+/// must decide between referencing an external file, or using a plain fragment identifier
+/// like `href="#foo"` as a reference to an SVG element in the same file as the one being
+/// processed.  This enum makes that distinction.
+#[derive(Debug, PartialEq)]
+pub enum Href {
+    PlainUrl(String),
+    WithFragment(Fragment),
+}
+
+/// Optional URI, mandatory fragment id
+#[derive(Debug, PartialEq, Clone)]
+pub struct Fragment(Option<String>, String);
+
+impl Fragment {
+    // Outside of testing, we don't want code creating Fragments by hand;
+    // they should get them from Href.
+    #[cfg(test)]
+    pub fn new(uri: Option<String>, fragment: String) -> Fragment {
+        Fragment(uri, fragment)
+    }
+
+    pub fn parse(href: &str) -> Result<Fragment, HrefError> {
+        match Href::parse(&href)? {
+            Href::PlainUrl(_) => Err(HrefError::FragmentRequired),
+            Href::WithFragment(f) => Ok(f),
+        }
+    }
+
+    pub fn uri(&self) -> Option<&str> {
+        self.0.as_ref().map(|s| s.as_str())
+    }
+
+    pub fn fragment(&self) -> &str {
+        &self.1
+    }
+}
+
+impl fmt::Display for Fragment {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        write!(
+            f,
+            "{}#{}",
+            self.0.as_ref().map(String::as_str).unwrap_or(""),
+            self.1
+        )
+    }
+}
+
+impl Href {
+    /// Parses a string into an Href, or returns an error
+    ///
+    /// An href can come from an `xlink:href` attribute in an SVG
+    /// element.  This function determines if the provided href is a
+    /// plain absolute or relative URL ("`foo.png`"), or one with a
+    /// fragment identifier ("`foo.svg#bar`").
+    pub fn parse(href: &str) -> Result<Href, HrefError> {
+        let (uri, fragment) = match href.rfind('#') {
+            None => (Some(href), None),
+            Some(p) if p == 0 => (None, Some(&href[1..])),
+            Some(p) => (Some(&href[..p]), Some(&href[(p + 1)..])),
+        };
+
+        match (uri, fragment) {
+            (None, Some(f)) if f.len() == 0 => Err(HrefError::ParseError),
+            (None, Some(f)) => Ok(Href::WithFragment(Fragment(None, f.to_string()))),
+            (Some(u), _) if u.len() == 0 => Err(HrefError::ParseError),
+            (Some(u), None) => Ok(Href::PlainUrl(u.to_string())),
+            (Some(_u), Some(f)) if f.len() == 0 => Err(HrefError::ParseError),
+            (Some(u), Some(f)) => Ok(Href::WithFragment(Fragment(
+                Some(u.to_string()),
+                f.to_string(),
+            ))),
+            (_, _) => Err(HrefError::ParseError),
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -200,8 +287,8 @@ mod tests {
         assert_eq!(
             AllowedUrl::from_href("data:image/jpeg;base64,xxyyzz", None)
                 .unwrap()
-                .url(),
-            &Url::parse("data:image/jpeg;base64,xxyyzz").unwrap(),
+                .as_ref(),
+            "data:image/jpeg;base64,xxyyzz",
         );
     }
 
@@ -213,8 +300,8 @@ mod tests {
                 Some(Url::parse("file:///example/bar.svg").unwrap()).as_ref()
             )
             .unwrap()
-            .url(),
-            &Url::parse("file:///example/foo.svg").unwrap(),
+            .as_ref(),
+            "file:///example/foo.svg",
         );
     }
 
@@ -226,8 +313,8 @@ mod tests {
                 Some(Url::parse("file:///example/bar.svg").unwrap()).as_ref()
             )
             .unwrap()
-            .url(),
-            &Url::parse("file:///example/foo.svg").unwrap(),
+            .as_ref(),
+            "file:///example/foo.svg",
         );
     }
 
@@ -239,8 +326,8 @@ mod tests {
                 Some(Url::parse("file:///example/bar.svg").unwrap()).as_ref()
             )
             .unwrap()
-            .url(),
-            &Url::parse("file:///example/subdir/foo.svg").unwrap(),
+            .as_ref(),
+            "file:///example/subdir/foo.svg",
         );
     }
 
@@ -254,4 +341,42 @@ mod tests {
             Err(AllowedUrlError::NotSiblingOrChildOfBaseFile)
         );
     }
+
+    #[test]
+    fn parses_href() {
+        assert_eq!(
+            Href::parse("uri").unwrap(),
+            Href::PlainUrl("uri".to_string())
+        );
+        assert_eq!(
+            Href::parse("#fragment").unwrap(),
+            Href::WithFragment(Fragment::new(None, "fragment".to_string()))
+        );
+        assert_eq!(
+            Href::parse("uri#fragment").unwrap(),
+            Href::WithFragment(Fragment::new(
+                Some("uri".to_string()),
+                "fragment".to_string()
+            ))
+        );
+
+        assert_eq!(Href::parse(""), Err(HrefError::ParseError));
+        assert_eq!(Href::parse("#"), Err(HrefError::ParseError));
+        assert_eq!(Href::parse("uri#"), Err(HrefError::ParseError));
+    }
+
+    #[test]
+    fn parses_fragment() {
+        assert_eq!(
+            Fragment::parse("#foo").unwrap(),
+            Fragment::new(None, "foo".to_string())
+        );
+
+        assert_eq!(
+            Fragment::parse("uri#foo").unwrap(),
+            Fragment::new(Some("uri".to_string()), "foo".to_string())
+        );
+
+        assert_eq!(Fragment::parse("uri"), Err(HrefError::FragmentRequired));
+    }
 }
