@@ -5,10 +5,10 @@ use glib::translate::*;
 use std::cell::RefCell;
 use std::rc::{Rc, Weak};
 
+use allowed_url::Fragment;
 use bbox::BoundingBox;
 use clip_path::{ClipPathUnits, NodeClipPath};
 use coord_units::CoordUnits;
-use defs::Fragment;
 use dpi::Dpi;
 use error::RenderingError;
 use filters;
@@ -18,10 +18,8 @@ use mask::NodeMask;
 use node::{CascadedValues, NodeType, RsvgNode};
 use paint_server::{PaintServer, PaintSource};
 use pattern::NodePattern;
-use rect::RectangleExt;
-use state::{
+use properties::{
     ClipRule,
-    CompOp,
     ComputedValues,
     EnableBackground,
     FillRule,
@@ -30,6 +28,8 @@ use state::{
     StrokeLinecap,
     StrokeLinejoin,
 };
+use rect::RectangleExt;
+use surface_utils::shared_surface::SharedImageSurface;
 use svg::Svg;
 use unit_interval::UnitInterval;
 use viewbox::ViewBox;
@@ -44,10 +44,10 @@ use viewbox::ViewBox;
 /// returned `ViewParams` is dropped; at that point, the `DrawingCtx` will resume using its
 /// previous viewport.
 pub struct ViewParams {
-    dpi_x: f64,
-    dpi_y: f64,
-    view_box_width: f64,
-    view_box_height: f64,
+    pub dpi_x: f64,
+    pub dpi_y: f64,
+    pub view_box_width: f64,
+    pub view_box_height: f64,
     view_box_stack: Option<Weak<RefCell<Vec<ViewBox>>>>,
 }
 
@@ -62,22 +62,6 @@ impl ViewParams {
             view_box_stack: None,
         }
     }
-
-    pub fn dpi_x(&self) -> f64 {
-        self.dpi_x
-    }
-
-    pub fn dpi_y(&self) -> f64 {
-        self.dpi_y
-    }
-
-    pub fn view_box_width(&self) -> f64 {
-        self.view_box_width
-    }
-
-    pub fn view_box_height(&self) -> f64 {
-        self.view_box_height
-    }
 }
 
 impl Drop for ViewParams {
@@ -113,8 +97,6 @@ pub struct DrawingCtx {
     cr: cairo::Context,
     initial_cr: cairo::Context,
 
-    surfaces_stack: Vec<cairo::ImageSurface>,
-
     view_box_stack: Rc<RefCell<Vec<ViewBox>>>,
 
     bbox: BoundingBox,
@@ -170,7 +152,6 @@ impl DrawingCtx {
             cr_stack: Vec::new(),
             cr: cr.clone(),
             initial_cr: cr.clone(),
-            surfaces_stack: Vec::new(),
             view_box_stack: Rc::new(RefCell::new(view_box_stack)),
             bbox: BoundingBox::new(&affine),
             bbox_stack: Vec::new(),
@@ -214,8 +195,8 @@ impl DrawingCtx {
         ViewParams {
             dpi_x: self.dpi.x(),
             dpi_y: self.dpi.y(),
-            view_box_width: stack_top.0.width,
-            view_box_height: stack_top.0.height,
+            view_box_width: stack_top.width,
+            view_box_height: stack_top.height,
             view_box_stack: None,
         }
     }
@@ -263,7 +244,7 @@ impl DrawingCtx {
     // acquire it again.  If you acquire a node "#foo" and don't release it before
     // trying to acquire "foo" again, you will obtain a %NULL the second time.
     pub fn get_acquired_node(&mut self, fragment: &Fragment) -> Option<AcquiredNode> {
-        if let Some(node) = self.svg.lookup(fragment) {
+        if let Ok(node) = self.svg.lookup(fragment) {
             if !self.acquired_nodes_contains(&node) {
                 self.acquired_nodes.borrow_mut().push(node.clone());
                 let acq = AcquiredNode(self.acquired_nodes.clone(), node.clone());
@@ -348,41 +329,45 @@ impl DrawingCtx {
             };
 
             let UnitInterval(opacity) = values.opacity.0;
-            let comp_op = values.comp_op;
             let enable_background = values.enable_background;
 
             let affine = original_cr.get_matrix();
 
-            let (acquired_clip, clip_units) = {
-                if let Some(acquired) = self.get_acquired_node_of_type(clip_uri, NodeType::ClipPath)
+            let (clip_in_user_space, clip_in_object_space) = {
+                if let Some(clip_node) =
+                    self.get_acquired_node_of_type(clip_uri, NodeType::ClipPath)
                 {
-                    let ClipPathUnits(units) = acquired
-                        .get()
-                        .with_impl(|clip_path: &NodeClipPath| clip_path.get_units());
+                    let clip_node = clip_node.get().clone();
 
-                    (Some(acquired), Some(units))
+                    let ClipPathUnits(units) =
+                        clip_node.with_impl(|clip_path: &NodeClipPath| clip_path.get_units());
+
+                    if units == CoordUnits::UserSpaceOnUse {
+                        (Some(clip_node), None)
+                    } else {
+                        assert!(units == CoordUnits::ObjectBoundingBox);
+                        (None, Some(clip_node))
+                    }
                 } else {
                     (None, None)
                 }
             };
 
-            if clip_units == Some(CoordUnits::UserSpaceOnUse) {
-                let clip_node = acquired_clip.as_ref().unwrap().get();
-                let res = clip_node.with_impl(|clip_path: &NodeClipPath| {
-                    clip_path.to_cairo_context(clip_node, &affine, self)
-                });
-
-                if let Err(e) = res {
-                    original_cr.restore();
-                    return Err(e);
-                }
+            if let Some(clip_node) = clip_in_user_space {
+                clip_node
+                    .with_impl(|clip_path: &NodeClipPath| {
+                        clip_path.to_cairo_context(&clip_node, &affine, self)
+                    })
+                    .map_err(|e| {
+                        original_cr.restore();
+                        e
+                    })?;
             }
 
             let needs_temporary_surface = !(opacity == 1.0
                 && filter.is_none()
                 && mask.is_none()
-                && (clip_units == None || clip_units == Some(CoordUnits::UserSpaceOnUse))
-                && comp_op == CompOp::SrcOver
+                && clip_in_object_space.is_none()
                 && enable_background == EnableBackground::Accumulate);
 
             if needs_temporary_surface {
@@ -392,10 +377,6 @@ impl DrawingCtx {
                     self.rect.height as i32,
                 )?;
 
-                if filter.is_some() {
-                    self.surfaces_stack.push(surface.clone());
-                }
-
                 let cr = cairo::Context::new(&surface);
                 cr.set_matrix(affine);
 
@@ -424,20 +405,17 @@ impl DrawingCtx {
                 original_cr.identity_matrix();
                 original_cr.set_source_surface(&filter_result_surface, xofs, yofs);
 
-                if clip_units == Some(CoordUnits::ObjectBoundingBox) {
-                    let clip_node = acquired_clip.as_ref().unwrap().get();
-                    let res = clip_node.with_impl(|clip_path: &NodeClipPath| {
-                        clip_path.to_cairo_context(clip_node, &affine, self)
-                    });
-
-                    if let Err(e) = res {
-                        original_cr.restore();
-                        return Err(e);
-                    }
+                if let Some(clip_node) = clip_in_object_space {
+                    clip_node
+                        .with_impl(|clip_path: &NodeClipPath| {
+                            clip_path.to_cairo_context(&clip_node, &affine, self)
+                        })
+                        .map_err(|e| {
+                            original_cr.restore();
+                            e
+                        })?;
                 }
 
-                original_cr.set_operator(cairo::Operator::from(comp_op));
-
                 if let Some(mask) = mask {
                     if let Some(acquired) =
                         self.get_acquired_node_of_type(Some(mask), NodeType::Mask)
@@ -480,15 +458,13 @@ impl DrawingCtx {
         values: &ComputedValues,
         child_surface: &cairo::ImageSurface,
     ) -> Result<cairo::ImageSurface, RenderingError> {
-        let output = self.surfaces_stack.pop().unwrap();
-
         match self.get_acquired_node_of_type(Some(filter_uri), NodeType::Filter) {
             Some(acquired) => {
                 let filter_node = acquired.get();
 
                 if !filter_node.is_in_error() {
                     // FIXME: deal with out of memory here
-                    filters::render(&filter_node, values, &output, self)
+                    filters::render(&filter_node, values, child_surface, self)
                 } else {
                     Ok(child_surface.clone())
                 }
@@ -713,6 +689,12 @@ impl DrawingCtx {
         }
     }
 
+    pub fn lookup_image(&self, href: &str) -> Result<SharedImageSurface, RenderingError> {
+        self.svg
+            .lookup_image(href)
+            .map_err(|_| RenderingError::InvalidHref)
+    }
+
     pub fn draw_node_on_surface(
         &mut self,
         node: &RsvgNode,
@@ -751,33 +733,26 @@ impl DrawingCtx {
         node: &RsvgNode,
         clipping: bool,
     ) -> Result<(), RenderingError> {
-        let mut draw = true;
-        let mut res = Ok(());
-
         let stack_top = self.drawsub_stack.pop();
 
-        if let Some(ref top) = stack_top {
-            if !Rc::ptr_eq(top, node) {
-                draw = false;
-            }
-        }
+        let draw = if let Some(ref top) = stack_top {
+            Rc::ptr_eq(top, node)
+        } else {
+            true
+        };
 
-        if draw {
-            let values = cascaded.get();
-            if values.is_visible() {
-                res = node.draw(node, cascaded, self, clipping);
-            }
-        }
+        let values = cascaded.get();
+        let res = if draw && values.is_visible() {
+            node.draw(node, cascaded, self, clipping)
+        } else {
+            Ok(())
+        };
 
         if let Some(top) = stack_top {
             self.drawsub_stack.push(top);
         }
 
-        if res.is_ok() {
-            res = self.check_limits();
-        }
-
-        res
+        res.and_then(|_| self.check_limits())
     }
 
     pub fn mask_surface(&mut self, mask: &cairo::ImageSurface) {
@@ -883,37 +858,6 @@ impl From<StrokeLinecap> for cairo::LineCap {
     }
 }
 
-impl From<CompOp> for cairo::Operator {
-    fn from(op: CompOp) -> cairo::Operator {
-        match op {
-            CompOp::Clear => cairo::Operator::Clear,
-            CompOp::Src => cairo::Operator::Source,
-            CompOp::Dst => cairo::Operator::Dest,
-            CompOp::SrcOver => cairo::Operator::Over,
-            CompOp::DstOver => cairo::Operator::DestOver,
-            CompOp::SrcIn => cairo::Operator::In,
-            CompOp::DstIn => cairo::Operator::DestIn,
-            CompOp::SrcOut => cairo::Operator::Out,
-            CompOp::DstOut => cairo::Operator::DestOut,
-            CompOp::SrcAtop => cairo::Operator::Atop,
-            CompOp::DstAtop => cairo::Operator::DestAtop,
-            CompOp::Xor => cairo::Operator::Xor,
-            CompOp::Plus => cairo::Operator::Add,
-            CompOp::Multiply => cairo::Operator::Multiply,
-            CompOp::Screen => cairo::Operator::Screen,
-            CompOp::Overlay => cairo::Operator::Overlay,
-            CompOp::Darken => cairo::Operator::Darken,
-            CompOp::Lighten => cairo::Operator::Lighten,
-            CompOp::ColorDodge => cairo::Operator::ColorDodge,
-            CompOp::ColorBurn => cairo::Operator::ColorBurn,
-            CompOp::HardLight => cairo::Operator::HardLight,
-            CompOp::SoftLight => cairo::Operator::SoftLight,
-            CompOp::Difference => cairo::Operator::Difference,
-            CompOp::Exclusion => cairo::Operator::Exclusion,
-        }
-    }
-}
-
 impl From<ClipRule> for cairo::FillRule {
     fn from(c: ClipRule) -> cairo::FillRule {
         match c {
