@@ -10,7 +10,6 @@ use glib_sys;
 use libc;
 
 use attributes::Attribute;
-use defs::HrefError;
 use parsers::ParseError;
 
 /// A simple error which refers to an attribute's value
@@ -91,20 +90,22 @@ impl<'a> From<BasicParseError<'a>> for ValueErrorKind {
     }
 }
 
-#[derive(Clone)]
+#[derive(Debug, Clone)]
 pub enum DefsLookupErrorKind {
     HrefError(HrefError),
     CannotLookupExternalReferences,
     NotFound,
 }
 
-#[derive(Clone)]
+#[derive(Debug, Clone)]
 pub enum RenderingError {
     Cairo(cairo::Status),
     CircularReference,
     InstancingLimit,
     InvalidId(DefsLookupErrorKind),
+    InvalidHref,
     SvgHasNoSize,
+    OutOfMemory,
 }
 
 impl From<cairo::Status> for RenderingError {
@@ -147,6 +148,38 @@ impl<O, E: Into<ValueErrorKind>> AttributeResultExt<O, E> for Result<O, E> {
     }
 }
 
+/// Errors returned when creating an `Href` out of a string
+#[derive(Debug, Clone, PartialEq)]
+pub enum HrefError {
+    /// The href is an invalid URI or has empty components.
+    ParseError,
+
+    /// A fragment identifier ("`#foo`") is not allowed here
+    ///
+    /// For example, the SVG `<image>` element only allows referencing
+    /// resources without fragment identifiers like
+    /// `xlink:href="foo.png"`.
+    FragmentForbidden,
+
+    /// A fragment identifier ("`#foo`") was required but not found.  For example,
+    /// the SVG `<use>` element requires one, as in `<use xlink:href="foo.svg#bar">`.
+    FragmentRequired,
+}
+
+impl From<HrefError> for ValueErrorKind {
+    fn from(e: HrefError) -> ValueErrorKind {
+        match e {
+            HrefError::ParseError => ValueErrorKind::Parse(ParseError::new("url parse error")),
+            HrefError::FragmentForbidden => {
+                ValueErrorKind::Value("fragment identifier not allowed".to_string())
+            }
+            HrefError::FragmentRequired => {
+                ValueErrorKind::Value("fragment identifier required".to_string())
+            }
+        }
+    }
+}
+
 #[derive(Debug, Clone)]
 pub enum LoadingError {
     NoDataPassedToParser,
@@ -202,6 +235,34 @@ impl fmt::Display for LoadingError {
     }
 }
 
+impl error::Error for RenderingError {
+    fn description(&self) -> &str {
+        match *self {
+            RenderingError::Cairo(_) => "cairo error",
+            RenderingError::CircularReference => "circular reference",
+            RenderingError::InstancingLimit => "instancing limit",
+            RenderingError::InvalidId(_) => "invalid id",
+            RenderingError::InvalidHref => "invalid href",
+            RenderingError::SvgHasNoSize => "svg has no size",
+            RenderingError::OutOfMemory => "out of memory",
+        }
+    }
+}
+
+impl fmt::Display for RenderingError {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match *self {
+            RenderingError::Cairo(ref status) => write!(f, "cairo error: {:?}", status),
+            RenderingError::InvalidId(ref id) => write!(f, "invalid id: {:?}", id),
+            RenderingError::CircularReference
+            | RenderingError::InstancingLimit
+            | RenderingError::InvalidHref
+            | RenderingError::SvgHasNoSize
+            | RenderingError::OutOfMemory => write!(f, "{}", self.description()),
+        }
+    }
+}
+
 impl From<cairo::Status> for LoadingError {
     fn from(e: cairo::Status) -> LoadingError {
         assert!(e != cairo::Status::Success);
@@ -260,7 +321,7 @@ pub fn is_value_error<T>(r: &Result<T, ValueErrorKind>) -> bool {
 pub struct RsvgError;
 
 // Keep in sync with rsvg.h:RsvgError
-const RSVG_ERROR_FAILED: i32 = 0;
+pub const RSVG_ERROR_FAILED: i32 = 0;
 
 impl ErrorDomain for RsvgError {
     fn domain() -> glib::Quark {
