@@ -3,7 +3,7 @@ use cairo::{self, MatrixTrait};
 
 use bbox::BoundingBox;
 use drawing_ctx::DrawingCtx;
-use length::Length;
+use length::{LengthHorizontal, LengthVertical};
 use rect::IRect;
 
 use super::context::{FilterContext, FilterInput, FilterOutput};
@@ -21,10 +21,10 @@ pub struct BoundsBuilder<'a> {
     standard_input_was_referenced: bool,
 
     /// Filter primitive properties.
-    x: Option<Length>,
-    y: Option<Length>,
-    width: Option<Length>,
-    height: Option<Length>,
+    x: Option<LengthHorizontal>,
+    y: Option<LengthVertical>,
+    width: Option<LengthHorizontal>,
+    height: Option<LengthVertical>,
 }
 
 impl<'a> BoundsBuilder<'a> {
@@ -32,10 +32,10 @@ impl<'a> BoundsBuilder<'a> {
     #[inline]
     pub fn new(
         ctx: &'a FilterContext,
-        x: Option<Length>,
-        y: Option<Length>,
-        width: Option<Length>,
-        height: Option<Length>,
+        x: Option<LengthHorizontal>,
+        y: Option<LengthVertical>,
+        width: Option<LengthHorizontal>,
+        height: Option<LengthVertical>,
     ) -> Self {
         Self {
             ctx,
@@ -115,24 +115,25 @@ impl<'a> BoundsBuilder<'a> {
 
         // If any of the properties were specified, we need to respect them.
         if self.x.is_some() || self.y.is_some() || self.width.is_some() || self.height.is_some() {
-            self.ctx.with_primitive_units(draw_ctx, |normalize| {
-                // These replacements are correct only because self.bbox is used with the paffine
-                // matrix.
-                let rect = self.bbox.rect.as_mut().unwrap();
+            let params = self.ctx.get_view_params(draw_ctx);
+            let values = self.ctx.get_computed_values_from_node_being_filtered();
 
-                if let Some(x) = self.x {
-                    rect.x = normalize(&x);
-                }
-                if let Some(y) = self.y {
-                    rect.y = normalize(&y);
-                }
-                if let Some(width) = self.width {
-                    rect.width = normalize(&width);
-                }
-                if let Some(height) = self.height {
-                    rect.height = normalize(&height);
-                }
-            });
+            // These replacements are correct only because self.bbox is used with the paffine
+            // matrix.
+            let rect = self.bbox.rect.as_mut().unwrap();
+
+            if let Some(x) = self.x {
+                rect.x = x.normalize(values, &params);
+            }
+            if let Some(y) = self.y {
+                rect.y = y.normalize(values, &params);
+            }
+            if let Some(width) = self.width {
+                rect.width = width.normalize(values, &params);
+            }
+            if let Some(height) = self.height {
+                rect.height = height.normalize(values, &params);
+            }
         }
 
         // Convert into the surface coordinate system.
