@@ -6,12 +6,11 @@ use cairo::{self, MatrixTrait};
 
 use bbox::BoundingBox;
 use coord_units::CoordUnits;
-use drawing_ctx::DrawingCtx;
-use length::Length;
+use drawing_ctx::{DrawingCtx, ViewParams};
 use node::RsvgNode;
 use paint_server::PaintServer;
+use properties::ComputedValues;
 use rect::IRect;
-use state::ComputedValues;
 use surface_utils::shared_surface::{SharedImageSurface, SurfaceType};
 use unit_interval::UnitInterval;
 
@@ -93,74 +92,6 @@ pub struct FilterContext {
     paffine: cairo::Matrix,
 }
 
-/// Computes and returns the filter effects region.
-fn compute_effects_region(
-    filter_node: &RsvgNode,
-    computed_from_target_node: &ComputedValues,
-    draw_ctx: &mut DrawingCtx,
-    affine: cairo::Matrix,
-    width: f64,
-    height: f64,
-) -> BoundingBox {
-    // Filters use the properties of the target node.
-    let values = computed_from_target_node;
-
-    let filter = filter_node.get_impl::<NodeFilter>().unwrap();
-
-    let mut bbox = BoundingBox::new(&cairo::Matrix::identity());
-
-    // affine is set up in FilterContext::new() in such a way that for
-    // filterunits == ObjectBoundingBox affine includes scaling to correct width, height and this
-    // is why width and height are set to 1, 1 (and for filterunits == UserSpaceOnUse affine
-    // doesn't include scaling because in this case the correct width, height already happens to be
-    // the viewbox width, height).
-    //
-    // It's done this way because with ObjectBoundingBox, non-percentage values are supposed to
-    // represent the fractions of the referenced node, and with width and height = 1, 1 this
-    // works out exactly like that.
-    let params = if filter.filterunits.get() == CoordUnits::ObjectBoundingBox {
-        draw_ctx.push_view_box(1.0, 1.0)
-    } else {
-        draw_ctx.get_view_params()
-    };
-
-    // With filterunits == ObjectBoundingBox, lengths represent fractions or percentages of the
-    // referencing node. No units are allowed (it's checked during attribute parsing).
-    let rect = if filter.filterunits.get() == CoordUnits::ObjectBoundingBox {
-        cairo::Rectangle {
-            x: filter.x.get().get_unitless(),
-            y: filter.y.get().get_unitless(),
-            width: filter.width.get().get_unitless(),
-            height: filter.height.get().get_unitless(),
-        }
-    } else {
-        cairo::Rectangle {
-            x: filter.x.get().normalize(values, &params),
-            y: filter.y.get().normalize(values, &params),
-            width: filter.width.get().normalize(values, &params),
-            height: filter.height.get().normalize(values, &params),
-        }
-    };
-
-    let other_bbox = BoundingBox::new(&affine).with_rect(Some(rect));
-
-    // At this point all of the previous viewbox and matrix business gets converted to pixel
-    // coordinates in the final surface, because bbox is created with an identity affine.
-    bbox.insert(&other_bbox);
-
-    // Finally, clip to the width and height of our surface.
-    let rect = cairo::Rectangle {
-        x: 0f64,
-        y: 0f64,
-        width,
-        height,
-    };
-    let other_bbox = BoundingBox::new(&cairo::Matrix::identity()).with_rect(Some(rect));
-    bbox.clip(&other_bbox);
-
-    bbox
-}
-
 impl FilterContext {
     /// Creates a new `FilterContext`.
     pub fn new(
@@ -223,8 +154,7 @@ impl FilterContext {
             last_result: None,
             previous_results: HashMap::new(),
             background_surface: UnsafeCell::new(None),
-            effects_region: compute_effects_region(
-                filter_node,
+            effects_region: filter.compute_effects_region(
                 computed_from_node_being_filtered,
                 draw_ctx,
                 affine,
@@ -366,28 +296,19 @@ impl FilterContext {
         self.effects_region
     }
 
-    /// Calls the given function with correct behavior for the value of `primitiveUnits`.
-    pub fn with_primitive_units<F, T>(&self, draw_ctx: &mut DrawingCtx, f: F) -> T
-    // TODO: Get rid of this Box? Can't just impl Trait because Rust cannot do higher-ranked types.
-    where
-        for<'b> F: FnOnce(Box<Fn(&Length) -> f64 + 'b>) -> T,
-    {
+    pub fn get_computed_from_node_being_filtered(&self) -> &ComputedValues {
+        &self.computed_from_node_being_filtered
+    }
+
+    /// Pushes the viewport size based on the value of `primitiveUnits`.
+    pub fn get_view_params(&self, draw_ctx: &mut DrawingCtx) -> ViewParams {
         let filter = self.node.get_impl::<NodeFilter>().unwrap();
 
         // See comments in compute_effects_region() for how this works.
         if filter.primitiveunits.get() == CoordUnits::ObjectBoundingBox {
-            let _params = draw_ctx.push_view_box(1.0, 1.0);
-            let rv = f(Box::new(Length::get_unitless));
-
-            rv
+            draw_ctx.push_view_box(1.0, 1.0)
         } else {
-            f(Box::new(|length: &Length| {
-                // Filters use the properties of the target node.
-                length.normalize(
-                    &self.computed_from_node_being_filtered,
-                    &draw_ctx.get_view_params(),
-                )
-            }))
+            draw_ctx.get_view_params()
         }
     }
 
