@@ -6,7 +6,6 @@ use cairo::{self, ImageSurface, MatrixTrait};
 use attributes::Attribute;
 use drawing_ctx::DrawingCtx;
 use error::NodeError;
-use handle::RsvgHandle;
 use node::{NodeResult, NodeTrait, RsvgNode};
 use parsers::{self, ParseError};
 use property_bag::PropertyBag;
@@ -49,13 +48,8 @@ impl Morphology {
 }
 
 impl NodeTrait for Morphology {
-    fn set_atts(
-        &self,
-        node: &RsvgNode,
-        handle: *const RsvgHandle,
-        pbag: &PropertyBag<'_>,
-    ) -> NodeResult {
-        self.base.set_atts(node, handle, pbag)?;
+    fn set_atts(&self, node: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
+        self.base.set_atts(node, pbag)?;
 
         for (attr, value) in pbag.iter() {
             match attr {
@@ -96,6 +90,10 @@ impl Filter for Morphology {
         let (rx, ry) = self.radius.get();
         let (rx, ry) = ctx.paffine().transform_distance(rx, ry);
 
+        // The radii can become negative here due to the transform.
+        let rx = rx.abs();
+        let ry = ry.abs();
+
         let operator = self.operator.get();
 
         let mut output_surface = ImageSurface::create(
