@@ -1,40 +1,37 @@
-use std::cell::{Cell, Ref, RefCell};
-use std::ffi::CString;
+use std;
+use std::cell::{Cell, RefCell};
+use std::ffi::{CStr, CString};
 use std::mem;
+use std::path::PathBuf;
 use std::ptr;
 use std::rc::Rc;
 use std::slice;
 
 use cairo::{self, ImageSurface, Status};
 use cairo_sys;
-use gdk_pixbuf::{Colorspace, Pixbuf, PixbufLoader, PixbufLoaderExt};
+use gdk_pixbuf::Pixbuf;
 use gdk_pixbuf_sys;
 use gio::{self, FileExt};
 use gio_sys;
 use glib::translate::*;
+use glib::{self, Bytes, Cast};
 use glib_sys;
 use gobject_sys;
 use libc;
 use url::Url;
 
-use allowed_url::AllowedUrl;
-use css::{self, CssStyles};
-use defs::Href;
+use allowed_url::{AllowedUrl, Href};
 use dpi::Dpi;
 use drawing_ctx::{DrawingCtx, RsvgRectangle};
 use error::{set_gerror, DefsLookupErrorKind, LoadingError, RenderingError};
-use io;
-use load::LoadContext;
-use node::{Node, RsvgNode};
-use rect::IRect;
+use node::RsvgNode;
+use pixbuf_utils::pixbuf_from_surface;
 use structure::NodeSvg;
-use surface_utils::{
-    iterators::Pixels,
-    shared_surface::SharedImageSurface,
-    shared_surface::SurfaceType,
-};
+use surface_utils::{shared_surface::SharedImageSurface, shared_surface::SurfaceType};
 use svg::Svg;
 use util::rsvg_g_warning;
+use xml::XmlState;
+use xml2_load::xml_state_load_from_possibly_compressed_stream;
 
 // A *const RsvgHandle is just an opaque pointer we get from C
 #[repr(C)]
@@ -45,17 +42,17 @@ pub struct RsvgHandle {
 // Keep in sync with rsvg.h:RsvgDimensionData
 #[repr(C)]
 pub struct RsvgDimensionData {
-    width: libc::c_int,
-    height: libc::c_int,
-    em: f64,
-    ex: f64,
+    pub width: libc::c_int,
+    pub height: libc::c_int,
+    pub em: f64,
+    pub ex: f64,
 }
 
 // Keep in sync with rsvg.h:RsvgPositionData
 #[repr(C)]
 pub struct RsvgPositionData {
-    x: libc::c_int,
-    y: libc::c_int,
+    pub x: libc::c_int,
+    pub y: libc::c_int,
 }
 
 /// Flags used during loading
@@ -81,6 +78,13 @@ impl LoadOptions {
     fn new(flags: LoadFlags, base_url: Option<Url>) -> LoadOptions {
         LoadOptions { flags, base_url }
     }
+
+    pub fn copy_with_base_url(&self, base_url: &AllowedUrl) -> LoadOptions {
+        LoadOptions {
+            flags: self.flags,
+            base_url: Some((*base_url).clone()),
+        }
+    }
 }
 
 #[derive(Copy, Clone, PartialEq)]
@@ -140,13 +144,13 @@ impl Drop for SizeCallback {
 }
 
 pub struct Handle {
-    dpi: Dpi,
+    dpi: Cell<Dpi>,
     base_url: RefCell<Option<Url>>,
     base_url_cstring: RefCell<Option<CString>>, // needed because the C api returns *const char
     svg: RefCell<Option<Rc<Svg>>>,
     load_flags: Cell<LoadFlags>,
     load_state: Cell<LoadState>,
-    load: RefCell<Option<LoadContext>>,
+    buffer: Vec<u8>, // used by the legacy write() api
     size_callback: RefCell<SizeCallback>,
     in_loop: Cell<bool>,
     is_testing: Cell<bool>,
@@ -155,13 +159,13 @@ pub struct Handle {
 impl Handle {
     fn new() -> Handle {
         Handle {
-            dpi: Dpi::default(),
+            dpi: Cell::new(Dpi::default()),
             base_url: RefCell::new(None),
             base_url_cstring: RefCell::new(None),
             svg: RefCell::new(None),
             load_flags: Cell::new(LoadFlags::default()),
             load_state: Cell::new(LoadState::Start),
-            load: RefCell::new(None),
+            buffer: Vec::new(),
             size_callback: RefCell::new(SizeCallback::new()),
             in_loop: Cell::new(false),
             is_testing: Cell::new(false),
@@ -205,14 +209,13 @@ impl Handle {
 
     pub fn read_stream_sync(
         &mut self,
-        handle: *mut RsvgHandle,
         stream: &gio::InputStream,
         cancellable: Option<&gio::Cancellable>,
     ) -> Result<(), LoadingError> {
         self.load_state.set(LoadState::Loading);
 
-        let svg = Svg::load_from_stream(&self.load_options(), handle, stream, cancellable)
-            .map_err(|e| {
+        let svg =
+            Svg::load_from_stream(&self.load_options(), stream, cancellable).map_err(|e| {
                 self.load_state.set(LoadState::ClosedError);
                 e
             })?;
@@ -226,42 +229,48 @@ impl Handle {
         LoadOptions::new(self.load_flags.get(), self.base_url.borrow().clone())
     }
 
-    pub fn write(&mut self, handle: *mut RsvgHandle, buf: &[u8]) {
-        assert!(
-            self.load_state.get() == LoadState::Start
-                || self.load_state.get() == LoadState::Loading
-        );
+    pub fn write(&mut self, buf: &[u8]) {
+        match self.load_state.get() {
+            LoadState::Start => self.load_state.set(LoadState::Loading),
+            LoadState::Loading => (),
+            _ => unreachable!(),
+        };
 
-        if self.load_state.get() == LoadState::Start {
-            self.load_state.set(LoadState::Loading);
-
-            self.load = RefCell::new(Some(LoadContext::new(handle, self.load_options())));
-        }
-
-        assert!(self.load_state.get() == LoadState::Loading);
-
-        self.load.borrow_mut().as_mut().unwrap().write(buf);
+        self.buffer.extend_from_slice(buf);
     }
 
     pub fn close(&mut self) -> Result<(), LoadingError> {
-        let load_state = self.load_state.get();
-
-        let res = match load_state {
+        let res = match self.load_state.get() {
             LoadState::Start => {
                 self.load_state.set(LoadState::ClosedError);
                 Err(LoadingError::NoDataPassedToParser)
             }
 
-            LoadState::Loading => self
-                .close_internal()
-                .and_then(|_| {
-                    self.load_state.set(LoadState::ClosedOk);
-                    Ok(())
-                })
+            LoadState::Loading => {
+                let bytes = Bytes::from(&self.buffer);
+                let stream = gio::MemoryInputStream::new_from_bytes(&bytes);
+                let mut xml = XmlState::new(&self.load_options());
+
+                xml_state_load_from_possibly_compressed_stream(
+                    &mut xml,
+                    self.load_flags.get(),
+                    &stream.upcast(),
+                    None,
+                )
                 .map_err(|e| {
                     self.load_state.set(LoadState::ClosedError);
                     e
-                }),
+                })?;
+
+                let svg = xml.steal_result().map_err(|e| {
+                    self.load_state.set(LoadState::ClosedError);
+                    e
+                })?;
+
+                self.load_state.set(LoadState::ClosedOk);
+                *self.svg.borrow_mut() = Some(Rc::new(svg));
+                Ok(())
+            }
 
             LoadState::ClosedOk | LoadState::ClosedError => {
                 // closing is idempotent
@@ -277,25 +286,6 @@ impl Handle {
         res
     }
 
-    fn close_internal(&mut self) -> Result<(), LoadingError> {
-        let mut r = self.load.borrow_mut();
-        let mut load = r.take().unwrap();
-
-        let mut xml = load.close()?;
-
-        xml.validate_tree()?;
-
-        *self.svg.borrow_mut() = Some(Rc::new(xml.steal_result()));
-        Ok(())
-    }
-
-    fn cascade(&mut self) {
-        let svg_ref = self.svg.borrow();
-        let svg = svg_ref.as_ref().unwrap();
-
-        svg.tree.cascade();
-    }
-
     fn create_drawing_ctx_for_node(
         &mut self,
         cr: &cairo::Context,
@@ -309,7 +299,7 @@ impl Handle {
             f64::from(dimensions.height),
             dimensions.em,
             dimensions.ex,
-            self.dpi.clone(),
+            self.dpi.get(),
             self.is_testing.get(),
         );
 
@@ -317,40 +307,91 @@ impl Handle {
             draw_ctx.add_node_and_ancestors_to_stack(node);
         }
 
-        self.cascade();
-
         draw_ctx
     }
 
-    fn get_dimensions(
-        &mut self,
-        handle: *mut RsvgHandle,
-    ) -> Result<RsvgDimensionData, RenderingError> {
-        let dimensions = unsafe {
-            let mut dimensions = mem::zeroed();
-            rsvg_handle_get_dimensions(handle, &mut dimensions);
-            dimensions
-        };
-
-        if dimensions.width == 0 || dimensions.height == 0 {
-            Err(RenderingError::SvgHasNoSize)
-        } else {
-            Ok(dimensions)
+    pub fn get_dimensions(&mut self) -> Result<RsvgDimensionData, RenderingError> {
+        // This function is probably called from the cairo_render functions,
+        // or is being erroneously called within the size_func.
+        // To prevent an infinite loop we are saving the state, and
+        // returning a meaningless size.
+        if self.in_loop.get() {
+            return Ok(RsvgDimensionData {
+                width: 1,
+                height: 1,
+                em: 1.0,
+                ex: 1.0,
+            });
         }
+
+        self.in_loop.set(true);
+
+        let res = self.get_dimensions_sub(None);
+
+        self.in_loop.set(false);
+
+        res.and_then(|dimensions| {
+            if dimensions.width == 0 || dimensions.height == 0 {
+                Err(RenderingError::SvgHasNoSize)
+            } else {
+                Ok(dimensions)
+            }
+        })
     }
 
+    fn get_dimensions_sub(
+        &mut self,
+        id: Option<&str>,
+    ) -> Result<RsvgDimensionData, RenderingError> {
+        let (ink_r, _) = self.get_geometry_sub(id)?;
+
+        let (w, h) = self
+            .size_callback
+            .borrow()
+            .call(ink_r.width as libc::c_int, ink_r.height as libc::c_int);
+
+        Ok(RsvgDimensionData {
+            width: w,
+            height: h,
+            em: ink_r.width,
+            ex: ink_r.height,
+        })
+    }
+
+    fn get_position_sub(&mut self, id: Option<&str>) -> Result<RsvgPositionData, RenderingError> {
+        if let None = id {
+            return Ok(RsvgPositionData { x: 0, y: 0 });
+        }
+
+        let (ink_r, _) = self.get_geometry_sub(id)?;
+
+        let width = ink_r.width as libc::c_int;
+        let height = ink_r.height as libc::c_int;
+
+        self.size_callback.borrow().call(width, height);
+
+        Ok(RsvgPositionData {
+            x: ink_r.x as libc::c_int,
+            y: ink_r.y as libc::c_int,
+        })
+    }
+
+    fn get_root(&self) -> RsvgNode {
+        let svg_ref = self.svg.borrow();
+        let svg = svg_ref.as_ref().unwrap();
+        svg.root()
+    }
+
+    /// Returns (ink_rect, logical_rect)
     fn get_node_geometry(
         &mut self,
-        handle: *mut RsvgHandle,
         node: &RsvgNode,
     ) -> Result<(RsvgRectangle, RsvgRectangle), RenderingError> {
-        let dimensions = self.get_dimensions(handle)?;
+        let dimensions = self.get_dimensions()?;
         let target = ImageSurface::create(cairo::Format::Rgb24, 1, 1)?;
         let cr = cairo::Context::new(&target);
         let mut draw_ctx = self.create_drawing_ctx_for_node(&cr, &dimensions, Some(node));
-        let svg_ref = self.svg.borrow();
-        let svg = svg_ref.as_ref().unwrap();
-        let root = svg.tree.root();
+        let root = self.get_root();
 
         draw_ctx.draw_node_from_stack(&root.get_cascaded_values(), &root, false)?;
 
@@ -368,17 +409,12 @@ impl Handle {
         Ok((ink_rect, logical_rect))
     }
 
+    /// Returns (ink_rect, logical_rect)
     fn get_geometry_sub(
         &mut self,
-        handle: *mut RsvgHandle,
         id: Option<&str>,
     ) -> Result<(RsvgRectangle, RsvgRectangle), RenderingError> {
-        let root = {
-            let svg_ref = self.svg.borrow();
-            let svg = svg_ref.as_ref().unwrap();
-
-            svg.tree.root()
-        };
+        let root = self.get_root();
 
         let (node, is_root) = if let Some(id) = id {
             let n = self.lookup_node(id).map_err(RenderingError::InvalidId)?;
@@ -390,7 +426,7 @@ impl Handle {
 
         if is_root {
             if let Some((root_width, root_height)) =
-                node.with_impl(|svg: &NodeSvg| svg.get_size(&self.dpi))
+                node.with_impl(|svg: &NodeSvg| svg.get_size(self.dpi.get()))
             {
                 let ink_r = RsvgRectangle {
                     x: 0.0,
@@ -405,16 +441,15 @@ impl Handle {
             }
         }
 
-        self.get_node_geometry(handle, &node)
+        self.get_node_geometry(&node)
     }
 
     fn lookup_node(&mut self, id: &str) -> Result<RsvgNode, DefsLookupErrorKind> {
         let svg_ref = self.svg.borrow();
         let svg = svg_ref.as_ref().unwrap();
 
-        let href = Href::with_fragment(id).map_err(DefsLookupErrorKind::HrefError)?;
-
-        match href {
+        match Href::parse(&id).map_err(DefsLookupErrorKind::HrefError)? {
+            Href::PlainUrl(_) => Err(DefsLookupErrorKind::CannotLookupExternalReferences),
             Href::WithFragment(fragment) => {
                 if let Some(uri) = fragment.uri() {
                     // The public APIs to get geometries of individual elements, or to render
@@ -443,19 +478,11 @@ impl Handle {
                     None => Err(DefsLookupErrorKind::NotFound),
                 }
             }
-
-            _ => unreachable!(), // we explicitly requested a with_fragment after all
         }
     }
 
-    fn has_sub(&mut self, name: &str) -> bool {
-        // FIXME: return a proper error; only NotFound should map to false
-        self.lookup_node(name).is_ok()
-    }
-
-    fn render_cairo_sub(
+    pub fn render_cairo_sub(
         &mut self,
-        handle: *mut RsvgHandle,
         cr: &cairo::Context,
         id: Option<&str>,
     ) -> Result<(), RenderingError> {
@@ -476,72 +503,28 @@ impl Handle {
             None
         };
 
-        let dimensions = self.get_dimensions(handle)?;
+        let dimensions = self.get_dimensions()?;
+        let root = self.get_root();
 
         cr.save();
-
         let mut draw_ctx = self.create_drawing_ctx_for_node(cr, &dimensions, node.as_ref());
-
-        let svg_ref = self.svg.borrow();
-        let svg = svg_ref.as_ref().unwrap();
-        let root = svg.tree.root();
-
         let res = draw_ctx.draw_node_from_stack(&root.get_cascaded_values(), &root, false);
-
         cr.restore();
 
         res
     }
 
-    fn get_pixbuf_sub(
+    fn construct_new_from_gfile_sync(
         &mut self,
-        handle: *mut RsvgHandle,
-        id: Option<&str>,
-    ) -> Result<Pixbuf, RenderingError> {
-        let dimensions = self.get_dimensions(handle)?;
-
-        let surface =
-            ImageSurface::create(cairo::Format::ARgb32, dimensions.width, dimensions.height)?;
-
-        {
-            let cr = cairo::Context::new(&surface);
-            self.render_cairo_sub(handle, &cr, id)?;
-        }
-
-        let surface = SharedImageSurface::new(surface, SurfaceType::SRgb)?;
-
-        let bounds = IRect {
-            x0: 0,
-            y0: 0,
-            x1: dimensions.width,
-            y1: dimensions.height,
-        };
-
-        let pixbuf = Pixbuf::new(
-            Colorspace::Rgb,
-            true,
-            8,
-            dimensions.width,
-            dimensions.height,
-        );
-
-        for (x, y, pixel) in Pixels::new(&surface, bounds) {
-            let (r, g, b, a) = if pixel.a == 0 {
-                (0, 0, 0, 0)
-            } else {
-                let pixel = pixel.unpremultiply();
-                (pixel.r, pixel.g, pixel.b, pixel.a)
-            };
-
-            pixbuf.put_pixel(x as i32, y as i32, r, g, b, a);
-        }
-
-        Ok(pixbuf)
+        file: &gio::File,
+        cancellable: Option<&gio::Cancellable>,
+    ) -> Result<(), LoadingError> {
+        let stream = file.read(cancellable)?;
+        self.construct_read_stream_sync(&stream.upcast(), Some(file), cancellable)
     }
 
     fn construct_read_stream_sync(
         &mut self,
-        handle: *mut RsvgHandle,
         stream: &gio::InputStream,
         base_file: Option<&gio::File>,
         cancellable: Option<&gio::Cancellable>,
@@ -550,7 +533,7 @@ impl Handle {
             self.set_base_gfile(file);
         }
 
-        self.read_stream_sync(handle, stream, cancellable)
+        self.read_stream_sync(stream, cancellable)
     }
 }
 
@@ -558,12 +541,6 @@ impl Handle {
 const RSVG_HANDLE_FLAG_UNLIMITED: u32 = 1 << 0;
 const RSVG_HANDLE_FLAG_KEEP_IMAGE_DATA: u32 = 1 << 1;
 
-pub fn get_load_options(handle: *const RsvgHandle) -> LoadOptions {
-    let rhandle = get_rust_handle(handle);
-
-    rhandle.load_options()
-}
-
 impl LoadFlags {
     pub fn from_flags(flags: u32) -> Self {
         LoadFlags {
@@ -589,151 +566,9 @@ impl LoadFlags {
 
 #[allow(improper_ctypes)]
 extern "C" {
-    fn rsvg_handle_new_with_flags(flags: u32) -> *mut RsvgHandle;
-
-    fn rsvg_handle_new_from_gfile_sync(
-        file: *const gio_sys::GFile,
-        flags: u32,
-        cancellable: *const gio_sys::GCancellable,
-        error: *mut *mut glib_sys::GError,
-    ) -> *mut RsvgHandle;
+    fn rsvg_handle_get_type() -> glib_sys::GType;
 
     fn rsvg_handle_get_rust(handle: *const RsvgHandle) -> *mut Handle;
-
-    fn rsvg_handle_get_dimensions(handle: *mut RsvgHandle, dimensions: *mut RsvgDimensionData);
-}
-
-// Looks up a node by its id.
-pub fn lookup_fragment_id(handle: *const RsvgHandle, id: &str) -> Option<Rc<Node>> {
-    let rhandle = get_rust_handle(handle);
-
-    let svg_ref = rhandle.svg.borrow();
-    let svg = svg_ref.as_ref().unwrap();
-
-    svg.lookup_node_by_id(id)
-}
-
-pub fn load_extern(load_options: &LoadOptions, aurl: &AllowedUrl) -> Result<*const RsvgHandle, ()> {
-    unsafe {
-        let file = gio::File::new_for_uri(aurl.url().as_str());
-
-        let res = rsvg_handle_new_from_gfile_sync(
-            file.to_glib_none().0,
-            load_options.flags.to_flags(),
-            ptr::null(),
-            ptr::null_mut(),
-        );
-
-        if res.is_null() {
-            Err(())
-        } else {
-            Ok(res)
-        }
-    }
-}
-
-pub fn get_base_url<'a>(handle: *const RsvgHandle) -> Ref<'a, Option<Url>> {
-    let rhandle = get_rust_handle(handle);
-
-    rhandle.base_url.borrow()
-}
-
-pub struct BinaryData {
-    pub data: Vec<u8>,
-    pub content_type: Option<String>,
-}
-
-pub fn load_image_to_surface(
-    load_options: &LoadOptions,
-    url: &str,
-) -> Result<ImageSurface, LoadingError> {
-    let aurl = AllowedUrl::from_href(url, load_options.base_url.as_ref())
-        .map_err(|_| LoadingError::BadUrl)?;
-
-    let data = io::acquire_data(&aurl, None)?;
-
-    if data.data.len() == 0 {
-        return Err(LoadingError::EmptyData);
-    }
-
-    let loader = if let Some(ref content_type) = data.content_type {
-        PixbufLoader::new_with_mime_type(content_type)?
-    } else {
-        PixbufLoader::new()
-    };
-
-    loader.write(&data.data)?;
-    loader.close()?;
-
-    let pixbuf = loader.get_pixbuf().ok_or(LoadingError::Unknown)?;
-
-    let surface = SharedImageSurface::from_pixbuf(&pixbuf)?.into_image_surface()?;
-
-    if load_options.flags.keep_image_data {
-        if let Some(mime_type) = data.content_type {
-            extern "C" {
-                fn cairo_surface_set_mime_data(
-                    surface: *mut cairo_sys::cairo_surface_t,
-                    mime_type: *const libc::c_char,
-                    data: *mut libc::c_char,
-                    length: libc::c_ulong,
-                    destroy: cairo_sys::cairo_destroy_func_t,
-                    closure: *mut libc::c_void,
-                ) -> Status;
-            }
-
-            let data_ptr = ToGlibContainerFromSlice::to_glib_full_from_slice(&data.data);
-
-            unsafe {
-                let status = cairo_surface_set_mime_data(
-                    surface.to_glib_none().0,
-                    mime_type.to_glib_none().0,
-                    data_ptr as *mut _,
-                    data.data.len() as libc::c_ulong,
-                    Some(glib_sys::g_free),
-                    data_ptr as *mut _,
-                );
-
-                if status != Status::Success {
-                    return Err(LoadingError::Cairo(status));
-                }
-            }
-        }
-    }
-
-    Ok(surface)
-}
-
-// This function just slurps CSS data from a possibly-relative href
-// and parses it.  We'll move it to a better place in the end.
-pub fn load_css(css_styles: &mut CssStyles, aurl: &AllowedUrl) -> Result<(), LoadingError> {
-    io::acquire_data(aurl, None)
-        .and_then(|data| {
-            let BinaryData {
-                data: bytes,
-                content_type,
-            } = data;
-
-            if content_type.as_ref().map(String::as_ref) == Some("text/css") {
-                Ok(bytes)
-            } else {
-                rsvg_log!("\"{}\" is not of type text/css; ignoring", aurl);
-                Err(LoadingError::BadCss)
-            }
-        })
-        .and_then(|bytes| {
-            String::from_utf8(bytes).map_err(|_| {
-                rsvg_log!(
-                    "\"{}\" does not contain valid UTF-8 CSS data; ignoring",
-                    aurl
-                );
-                LoadingError::BadCss
-            })
-        })
-        .and_then(|utf8| {
-            css::parse_into_css_styles(css_styles, Some(aurl.url().clone()), &utf8);
-            Ok(()) // FIXME: return CSS parsing errors
-        })
 }
 
 #[no_mangle]
@@ -747,21 +582,21 @@ pub unsafe extern "C" fn rsvg_handle_rust_free(raw_handle: *mut Handle) {
     Box::from_raw(raw_handle);
 }
 
-fn get_rust_handle<'a>(handle: *const RsvgHandle) -> &'a mut Handle {
+pub fn get_rust_handle<'a>(handle: *const RsvgHandle) -> &'a mut Handle {
     unsafe { &mut *(rsvg_handle_get_rust(handle) as *mut Handle) }
 }
 
 #[no_mangle]
 pub unsafe extern "C" fn rsvg_handle_rust_set_base_url(
-    raw_handle: *const Handle,
+    raw_handle: *mut RsvgHandle,
     uri: *const libc::c_char,
 ) {
-    let handle = &*raw_handle;
+    let rhandle = get_rust_handle(raw_handle);
 
     assert!(!uri.is_null());
     let uri: String = from_glib_none(uri);
 
-    handle.set_base_url(&uri);
+    rhandle.set_base_url(&uri);
 }
 
 #[no_mangle]
@@ -778,80 +613,80 @@ pub unsafe extern "C" fn rsvg_handle_rust_get_base_gfile(
 
 #[no_mangle]
 pub unsafe extern "C" fn rsvg_handle_rust_set_base_gfile(
-    raw_handle: *const Handle,
+    raw_handle: *mut RsvgHandle,
     raw_gfile: *mut gio_sys::GFile,
 ) {
-    let handle = &*raw_handle;
+    let rhandle = get_rust_handle(raw_handle);
 
     assert!(!raw_gfile.is_null());
 
     let file: gio::File = from_glib_none(raw_gfile);
 
-    handle.set_base_gfile(&file);
+    rhandle.set_base_gfile(&file);
 }
 
 #[no_mangle]
 pub unsafe extern "C" fn rsvg_handle_rust_get_base_url(
-    raw_handle: *const Handle,
+    raw_handle: *const RsvgHandle,
 ) -> *const libc::c_char {
-    let handle = &*raw_handle;
+    let rhandle = get_rust_handle(raw_handle);
 
-    match *handle.base_url_cstring.borrow() {
+    match *rhandle.base_url_cstring.borrow() {
         None => ptr::null(),
         Some(ref url) => url.as_ptr(),
     }
 }
 
 #[no_mangle]
-pub unsafe extern "C" fn rsvg_handle_rust_set_dpi_x(raw_handle: *const Handle, dpi_x: f64) {
-    let handle = &*(raw_handle as *const Handle);
+pub unsafe extern "C" fn rsvg_handle_rust_set_dpi_x(raw_handle: *mut RsvgHandle, dpi_x: f64) {
+    let rhandle = get_rust_handle(raw_handle);
 
-    handle.dpi.set_x(dpi_x);
+    rhandle.dpi.set(Dpi::new(dpi_x, rhandle.dpi.get().y()));
 }
 
 #[no_mangle]
-pub unsafe extern "C" fn rsvg_handle_rust_get_dpi_x(raw_handle: *const Handle) -> f64 {
-    let handle = &*(raw_handle as *const Handle);
+pub unsafe extern "C" fn rsvg_handle_rust_get_dpi_x(raw_handle: *const RsvgHandle) -> f64 {
+    let rhandle = get_rust_handle(raw_handle);
 
-    handle.dpi.x()
+    rhandle.dpi.get().x()
 }
 
 #[no_mangle]
-pub unsafe extern "C" fn rsvg_handle_rust_set_dpi_y(raw_handle: *const Handle, dpi_y: f64) {
-    let handle = &*(raw_handle as *const Handle);
+pub unsafe extern "C" fn rsvg_handle_rust_set_dpi_y(raw_handle: *mut RsvgHandle, dpi_y: f64) {
+    let rhandle = get_rust_handle(raw_handle);
 
-    handle.dpi.set_y(dpi_y);
+    rhandle.dpi.set(Dpi::new(rhandle.dpi.get().x(), dpi_y));
 }
 
 #[no_mangle]
-pub unsafe extern "C" fn rsvg_handle_rust_get_dpi_y(raw_handle: *const Handle) -> f64 {
-    let handle = &*(raw_handle as *const Handle);
+pub unsafe extern "C" fn rsvg_handle_rust_get_dpi_y(raw_handle: *const RsvgHandle) -> f64 {
+    let rhandle = get_rust_handle(raw_handle);
 
-    handle.dpi.y()
+    rhandle.dpi.get().y()
 }
 
 #[no_mangle]
-pub unsafe extern "C" fn rsvg_handle_rust_get_flags(raw_handle: *const Handle) -> u32 {
-    let rhandle = &*raw_handle;
+pub unsafe extern "C" fn rsvg_handle_rust_get_flags(raw_handle: *const RsvgHandle) -> u32 {
+    let rhandle = get_rust_handle(raw_handle);
 
     rhandle.load_flags.get().to_flags()
 }
 
 #[no_mangle]
-pub unsafe extern "C" fn rsvg_handle_rust_set_flags(raw_handle: *const Handle, flags: u32) {
-    let rhandle = &*raw_handle;
+pub unsafe extern "C" fn rsvg_handle_rust_set_flags(raw_handle: *const RsvgHandle, flags: u32) {
+    let rhandle = get_rust_handle(raw_handle);
 
     rhandle.load_flags.set(LoadFlags::from_flags(flags));
 }
 
 #[no_mangle]
 pub unsafe extern "C" fn rsvg_handle_rust_set_size_callback(
-    raw_handle: *mut Handle,
+    raw_handle: *mut RsvgHandle,
     size_func: RsvgSizeFunc,
     user_data: glib_sys::gpointer,
     destroy_notify: glib_sys::GDestroyNotify,
 ) {
-    let rhandle = &mut *raw_handle;
+    let rhandle = get_rust_handle(raw_handle);
 
     *rhandle.size_callback.borrow_mut() = SizeCallback {
         size_func,
@@ -862,10 +697,10 @@ pub unsafe extern "C" fn rsvg_handle_rust_set_size_callback(
 
 #[no_mangle]
 pub unsafe extern "C" fn rsvg_handle_rust_set_testing(
-    raw_handle: *const Handle,
+    raw_handle: *mut RsvgHandle,
     testing: glib_sys::gboolean,
 ) {
-    let rhandle = &*raw_handle;
+    let rhandle = get_rust_handle(raw_handle);
 
     rhandle.is_testing.set(from_glib(testing));
 }
@@ -913,7 +748,7 @@ pub unsafe extern "C" fn rsvg_handle_rust_read_stream_sync(
     let stream = from_glib_none(stream);
     let cancellable: Option<gio::Cancellable> = from_glib_none(cancellable);
 
-    match rhandle.read_stream_sync(handle, &stream, cancellable.as_ref()) {
+    match rhandle.read_stream_sync(&stream, cancellable.as_ref()) {
         Ok(()) => true.to_glib(),
 
         Err(e) => {
@@ -940,7 +775,7 @@ pub unsafe extern "C" fn rsvg_handle_rust_write(
 
     let buffer = slice::from_raw_parts(buf, count);
 
-    rhandle.write(handle, buffer);
+    rhandle.write(buffer);
 }
 
 #[no_mangle]
@@ -975,7 +810,7 @@ pub unsafe extern "C" fn rsvg_handle_rust_get_geometry_sub(
 
     let id: Option<String> = from_glib_none(id);
 
-    match rhandle.get_geometry_sub(handle, id.as_ref().map(String::as_str)) {
+    match rhandle.get_geometry_sub(id.as_ref().map(String::as_str)) {
         Ok((ink_r, logical_r)) => {
             if !out_ink_rect.is_null() {
                 *out_ink_rect = ink_r;
@@ -1019,7 +854,7 @@ pub unsafe extern "C" fn rsvg_handle_rust_has_sub(
     }
 
     let id: String = from_glib_none(id);
-    rhandle.has_sub(&id).to_glib()
+    rhandle.lookup_node(&id).is_ok().to_glib()
 }
 
 #[no_mangle]
@@ -1036,7 +871,7 @@ pub unsafe extern "C" fn rsvg_handle_rust_render_cairo_sub(
         return false.to_glib();
     }
 
-    match rhandle.render_cairo_sub(handle, &cr, id.as_ref().map(String::as_str)) {
+    match rhandle.render_cairo_sub(&cr, id.as_ref().map(String::as_str)) {
         Ok(()) => true.to_glib(),
 
         Err(_) => {
@@ -1046,6 +881,21 @@ pub unsafe extern "C" fn rsvg_handle_rust_render_cairo_sub(
     }
 }
 
+fn get_pixbuf_sub(handle: &mut Handle, id: Option<&str>) -> Result<Pixbuf, RenderingError> {
+    let dimensions = handle.get_dimensions()?;
+
+    let surface = ImageSurface::create(cairo::Format::ARgb32, dimensions.width, dimensions.height)?;
+
+    {
+        let cr = cairo::Context::new(&surface);
+        handle.render_cairo_sub(&cr, id)?;
+    }
+
+    let surface = SharedImageSurface::new(surface, SurfaceType::SRgb)?;
+
+    pixbuf_from_surface(&surface)
+}
+
 #[no_mangle]
 pub unsafe extern "C" fn rsvg_handle_rust_get_pixbuf_sub(
     handle: *mut RsvgHandle,
@@ -1058,7 +908,7 @@ pub unsafe extern "C" fn rsvg_handle_rust_get_pixbuf_sub(
         return ptr::null_mut();
     }
 
-    match rhandle.get_pixbuf_sub(handle, id.as_ref().map(String::as_str)) {
+    match get_pixbuf_sub(rhandle, id.as_ref().map(String::as_str)) {
         Ok(pixbuf) => pixbuf.to_glib_full(),
         Err(_) => ptr::null_mut(),
     }
@@ -1075,20 +925,22 @@ pub unsafe extern "C" fn rsvg_handle_rust_get_dimensions(
         return;
     }
 
-    // This function is probably called from the cairo_render functions.
-    // To prevent an infinite loop we are saving the state.
-    if rhandle.in_loop.get() {
-        // Called within the size function, so return a standard size
-        (*dimension_data).width = 1;
-        (*dimension_data).height = 1;
-        (*dimension_data).em = 1.0;
-        (*dimension_data).ex = 1.0;
-        return;
-    }
+    match rhandle.get_dimensions() {
+        Ok(dimensions) => {
+            *dimension_data = dimensions;
+        }
 
-    rhandle.in_loop.set(true);
-    rsvg_handle_rust_get_dimensions_sub(handle, dimension_data, ptr::null());
-    rhandle.in_loop.set(false);
+        Err(_) => {
+            let d = &mut *dimension_data;
+
+            d.width = 0;
+            d.height = 0;
+            d.em = 0.0;
+            d.ex = 0.0;
+
+            // This old API doesn't even let us return an error, sigh.
+        }
+    }
 }
 
 #[no_mangle]
@@ -1103,38 +955,32 @@ pub unsafe extern "C" fn rsvg_handle_rust_get_dimensions_sub(
         return false.to_glib();
     }
 
-    let mut ink_r = RsvgRectangle {
-        x: 0.0,
-        y: 0.0,
-        width: 0.0,
-        height: 0.0,
-    };
+    let id: Option<String> = from_glib_none(id);
 
-    let res = rsvg_handle_rust_get_geometry_sub(handle, &mut ink_r, ptr::null_mut(), id);
-    if from_glib(res) {
-        let (w, h) = rhandle
-            .size_callback
-            .borrow()
-            .call(ink_r.width as libc::c_int, ink_r.height as libc::c_int);
+    match rhandle.get_dimensions_sub(id.as_ref().map(String::as_str)) {
+        Ok(dimensions) => {
+            *dimension_data = dimensions;
+            true.to_glib()
+        }
 
-        (*dimension_data).width = w;
-        (*dimension_data).height = h;
-        (*dimension_data).em = ink_r.width;
-        (*dimension_data).ex = ink_r.height;
-    } else {
-        (*dimension_data).width = 0;
-        (*dimension_data).height = 0;
-        (*dimension_data).em = 0.0;
-        (*dimension_data).ex = 0.0;
+        Err(_) => {
+            let d = &mut *dimension_data;
+
+            d.width = 0;
+            d.height = 0;
+            d.em = 0.0;
+            d.ex = 0.0;
+
+            // FIXME: return a proper error code to the public API
+            false.to_glib()
+        }
     }
-
-    res
 }
 
 #[no_mangle]
 pub unsafe extern "C" fn rsvg_handle_rust_get_position_sub(
     handle: *mut RsvgHandle,
-    position: *mut RsvgPositionData,
+    position_data: *mut RsvgPositionData,
     id: *const libc::c_char,
 ) -> glib_sys::gboolean {
     let rhandle = get_rust_handle(handle);
@@ -1143,35 +989,80 @@ pub unsafe extern "C" fn rsvg_handle_rust_get_position_sub(
         return false.to_glib();
     }
 
-    // Short-cut when no id is given
-    if id.is_null() || *id == 0 {
-        (*position).x = 0;
-        (*position).y = 0;
-        return true.to_glib();
+    let id: Option<String> = from_glib_none(id);
+
+    match rhandle.get_position_sub(id.as_ref().map(String::as_str)) {
+        Ok(position) => {
+            *position_data = position;
+            true.to_glib()
+        }
+
+        Err(_) => {
+            let p = &mut *position_data;
+
+            p.x = 0;
+            p.y = 0;
+
+            // FIXME: return a proper error code to the public API
+            false.to_glib()
+        }
     }
+}
 
-    let mut ink_r = RsvgRectangle {
-        x: 0.0,
-        y: 0.0,
-        width: 0.0,
-        height: 0.0,
-    };
+#[no_mangle]
+pub unsafe extern "C" fn rsvg_handle_rust_new_with_flags(flags: u32) -> *mut RsvgHandle {
+    let obj: *mut gobject_sys::GObject =
+        glib::Object::new(from_glib(rsvg_handle_get_type()), &[("flags", &flags)])
+            .unwrap()
+            .to_glib_full();
 
-    let res = rsvg_handle_rust_get_geometry_sub(handle, &mut ink_r, ptr::null_mut(), id);
-    if from_glib(res) {
-        (*position).x = ink_r.x as libc::c_int;
-        (*position).y = ink_r.y as libc::c_int;
+    obj as *mut _
+}
 
-        let width = ink_r.width as libc::c_int;
-        let height = ink_r.height as libc::c_int;
+#[no_mangle]
+pub unsafe extern "C" fn rsvg_handle_rust_new_from_file(
+    filename: *const libc::c_char,
+    error: *mut *mut glib_sys::GError,
+) -> *mut RsvgHandle {
+    // This API lets the caller pass a URI, or a file name in the operating system's
+    // encoding.  So, first we'll see if it's UTF-8, and in that case, try the URL version.
+    // Otherwise, we'll try building a path name.
 
-        rhandle.size_callback.borrow().call(width, height);
-    } else {
-        (*position).x = 0;
-        (*position).y = 0;
+    let cstr = CStr::from_ptr(filename);
+
+    let file = cstr
+        .to_str()
+        .map_err(|_| ())
+        .and_then(|utf8| Url::parse(utf8).map_err(|_| ()))
+        .and_then(|url| Ok(gio::File::new_for_uri(url.as_str())))
+        .unwrap_or_else(|_| gio::File::new_for_path(PathBuf::from_glib_none(filename)));
+
+    rsvg_handle_rust_new_from_gfile_sync(file.to_glib_none().0, 0, ptr::null_mut(), error)
+}
+
+#[no_mangle]
+pub unsafe extern "C" fn rsvg_handle_rust_new_from_gfile_sync(
+    file: *mut gio_sys::GFile,
+    flags: u32,
+    cancellable: *mut gio_sys::GCancellable,
+    error: *mut *mut glib_sys::GError,
+) -> *mut RsvgHandle {
+    let raw_handle = rsvg_handle_rust_new_with_flags(flags);
+
+    let rhandle = get_rust_handle(raw_handle);
+
+    let file = from_glib_none(file);
+    let cancellable: Option<gio::Cancellable> = from_glib_none(cancellable);
+
+    match rhandle.construct_new_from_gfile_sync(&file, cancellable.as_ref()) {
+        Ok(()) => raw_handle,
+
+        Err(e) => {
+            set_gerror(error, 0, &format!("{}", e));
+            gobject_sys::g_object_unref(raw_handle as *mut _);
+            ptr::null_mut()
+        }
     }
-
-    res
 }
 
 #[no_mangle]
@@ -1182,7 +1073,7 @@ pub unsafe extern "C" fn rsvg_handle_rust_new_from_stream_sync(
     cancellable: *mut gio_sys::GCancellable,
     error: *mut *mut glib_sys::GError,
 ) -> *mut RsvgHandle {
-    let raw_handle = rsvg_handle_new_with_flags(flags);
+    let raw_handle = rsvg_handle_rust_new_with_flags(flags);
 
     let rhandle = get_rust_handle(raw_handle);
 
@@ -1190,12 +1081,7 @@ pub unsafe extern "C" fn rsvg_handle_rust_new_from_stream_sync(
     let stream = from_glib_none(input_stream);
     let cancellable: Option<gio::Cancellable> = from_glib_none(cancellable);
 
-    match rhandle.construct_read_stream_sync(
-        raw_handle,
-        &stream,
-        base_file.as_ref(),
-        cancellable.as_ref(),
-    ) {
+    match rhandle.construct_read_stream_sync(&stream, base_file.as_ref(), cancellable.as_ref()) {
         Ok(()) => raw_handle,
 
         Err(e) => {
@@ -1205,3 +1091,36 @@ pub unsafe extern "C" fn rsvg_handle_rust_new_from_stream_sync(
         }
     }
 }
+
+#[no_mangle]
+pub unsafe extern "C" fn rsvg_handle_rust_new_from_data(
+    data: *mut u8,
+    len: usize,
+    error: *mut *mut glib_sys::GError,
+) -> *mut RsvgHandle {
+    // We create the MemoryInputStream without the gtk-rs binding because of this:
+    //
+    // - The binding doesn't provide _new_from_data().  All of the binding's ways to
+    // put data into a MemoryInputStream involve copying the data buffer.
+    //
+    // - We can't use glib::Bytes from the binding either, for the same reason.
+    //
+    // - For now, we are using the other C-visible constructor, so we need a raw pointer to the
+    //   stream, anyway.
+
+    assert!(len <= std::isize::MAX as usize);
+    let len = len as isize;
+
+    let raw_stream = gio_sys::g_memory_input_stream_new_from_data(data, len, None);
+
+    let ret = rsvg_handle_rust_new_from_stream_sync(
+        raw_stream as *mut _,
+        ptr::null_mut(), // base_file
+        0,               // flags
+        ptr::null_mut(), // cancellable
+        error,
+    );
+
+    gobject_sys::g_object_unref(raw_stream as *mut _);
+    ret
+}
