@@ -1,6 +1,6 @@
 use cssparser::Parser;
 
-use defs::{Fragment, Href};
+use allowed_url::{Fragment, Href};
 use parsers::Parse;
 use parsers::ParseError;
 
@@ -33,10 +33,9 @@ impl IRI {
 }
 
 impl Parse for IRI {
-    type Data = ();
     type Err = ParseError;
 
-    fn parse(parser: &mut Parser<'_, '_>, _: Self::Data) -> Result<IRI, ParseError> {
+    fn parse(parser: &mut Parser<'_, '_>) -> Result<IRI, ParseError> {
         if parser.try(|i| i.expect_ident_matching("none")).is_ok() {
             Ok(IRI::None)
         } else {
@@ -48,13 +47,9 @@ impl Parse for IRI {
                 .expect_exhausted()
                 .map_err(|_| ParseError::new("expected url"))?;
 
-            let href =
-                Href::with_fragment(&url).map_err(|_| ParseError::new("could not parse href"))?;
-
-            if let Href::WithFragment(fragment) = href {
-                Ok(IRI::Resource(fragment))
-            } else {
-                Err(ParseError::new("href requires a fragment identifier"))
+            match Href::parse(&url).map_err(|_| ParseError::new("could not parse href"))? {
+                Href::PlainUrl(_) => Err(ParseError::new("href requires a fragment identifier")),
+                Href::WithFragment(f) => Ok(IRI::Resource(f)),
             }
         }
     }
@@ -66,18 +61,18 @@ mod tests {
 
     #[test]
     fn parses_none() {
-        assert_eq!(IRI::parse_str("none", ()), Ok(IRI::None));
+        assert_eq!(IRI::parse_str("none"), Ok(IRI::None));
     }
 
     #[test]
     fn parses_url() {
         assert_eq!(
-            IRI::parse_str("url(#bar)", ()),
+            IRI::parse_str("url(#bar)"),
             Ok(IRI::Resource(Fragment::new(None, "bar".to_string())))
         );
 
         assert_eq!(
-            IRI::parse_str("url(foo#bar)", ()),
+            IRI::parse_str("url(foo#bar)"),
             Ok(IRI::Resource(Fragment::new(
                 Some("foo".to_string()),
                 "bar".to_string()
@@ -86,19 +81,19 @@ mod tests {
 
         // be permissive if the closing ) is missing
         assert_eq!(
-            IRI::parse_str("url(#bar", ()),
+            IRI::parse_str("url(#bar"),
             Ok(IRI::Resource(Fragment::new(None, "bar".to_string())))
         );
         assert_eq!(
-            IRI::parse_str("url(foo#bar", ()),
+            IRI::parse_str("url(foo#bar"),
             Ok(IRI::Resource(Fragment::new(
                 Some("foo".to_string()),
                 "bar".to_string()
             )))
         );
 
-        assert!(IRI::parse_str("", ()).is_err());
-        assert!(IRI::parse_str("foo", ()).is_err());
-        assert!(IRI::parse_str("url(foo)bar", ()).is_err());
+        assert!(IRI::parse_str("").is_err());
+        assert!(IRI::parse_str("foo").is_err());
+        assert!(IRI::parse_str("url(foo)bar").is_err());
     }
 }
