@@ -3,8 +3,9 @@ use std::f64::consts::*;
 
 use drawing_ctx::ViewParams;
 use error::*;
-use parsers::{Parse, ParseError, finite_f32};
-use state::ComputedValues;
+use parsers::Parse;
+use parsers::ParseError;
+use properties::ComputedValues;
 
 #[derive(Debug, PartialEq, Copy, Clone)]
 pub enum LengthUnit {
@@ -16,29 +17,130 @@ pub enum LengthUnit {
 }
 
 #[derive(Debug, PartialEq, Copy, Clone)]
-pub enum LengthDir {
+enum LengthDir {
     Horizontal,
     Vertical,
     Both,
 }
 
-#[derive(Debug, PartialEq, Copy, Clone)]
-pub struct Length {
-    pub length: f64,
-    pub unit: LengthUnit,
-    dir: LengthDir,
-}
-
-impl Default for Length {
-    fn default() -> Length {
-        Length {
-            length: 0.0,
-            unit: LengthUnit::Default,
-            dir: LengthDir::Both,
+impl LengthDir {
+    fn scaling_factor(&self, x: f64, y: f64) -> f64 {
+        match *self {
+            LengthDir::Horizontal => x,
+            LengthDir::Vertical => y,
+            LengthDir::Both => viewport_percentage(x, y),
         }
     }
 }
 
+macro_rules! define_length_type {
+    ($name:ident, $dir:expr) => {
+        #[derive(Debug, PartialEq, Copy, Clone)]
+        pub struct $name(Length);
+
+        impl $name {
+            pub fn new(length: f64, unit: LengthUnit) -> Self {
+                $name(Length::new(length, unit))
+            }
+
+            pub fn length(&self) -> f64 {
+                self.0.length
+            }
+
+            pub fn unit(&self) -> LengthUnit {
+                self.0.unit
+            }
+
+            pub fn get_unitless(&self) -> f64 {
+                self.0.get_unitless()
+            }
+
+            pub fn check_nonnegative(self) -> Result<Self, ValueErrorKind> {
+                if self.length() >= 0.0 {
+                    Ok(self)
+                } else {
+                    Err(ValueErrorKind::Value(
+                        "value must be non-negative".to_string(),
+                    ))
+                }
+            }
+
+            pub fn normalize(&self, values: &ComputedValues, params: &ViewParams) -> f64 {
+                match self.unit() {
+                    LengthUnit::Default => self.length(),
+
+                    LengthUnit::Percent => {
+                        self.length()
+                            * $dir.scaling_factor(params.view_box_width, params.view_box_height)
+                    }
+
+                    LengthUnit::FontEm => self.length() * font_size_from_values(values, params),
+
+                    LengthUnit::FontEx => {
+                        self.length() * font_size_from_values(values, params) / 2.0
+                    }
+
+                    LengthUnit::Inch => {
+                        self.length() * $dir.scaling_factor(params.dpi_x, params.dpi_y)
+                    }
+                }
+            }
+
+            pub fn hand_normalize(
+                &self,
+                pixels_per_inch: f64,
+                width_or_height: f64,
+                font_size: f64,
+            ) -> f64 {
+                self.0
+                    .hand_normalize(pixels_per_inch, width_or_height, font_size)
+            }
+
+            pub fn from_cssparser(parser: &mut Parser<'_, '_>) -> Result<Self, ValueErrorKind> {
+                Ok($name(Length::from_cssparser(parser)?))
+            }
+        }
+
+        impl Default for $name {
+            fn default() -> Self {
+                $name(Length::new(0.0, LengthUnit::Default))
+            }
+        }
+
+        impl Parse for $name {
+            type Err = ValueErrorKind;
+
+            fn parse(parser: &mut Parser<'_, '_>) -> Result<$name, ValueErrorKind> {
+                Ok($name(Length::parse(parser)?))
+            }
+        }
+    };
+}
+
+/// Horizontal length
+///
+/// When this is specified as a percent value, it will get resolved
+/// against the current viewport's width.
+define_length_type!(LengthHorizontal, LengthDir::Horizontal);
+
+/// Vertical length
+///
+/// When this is specified as a percent value, it will get resolved
+/// against the current viewport's height.
+define_length_type!(LengthVertical, LengthDir::Vertical);
+
+/// "Both" length
+///
+/// When this is specified as a percent value, it will get resolved
+/// against the current viewport's width and height.
+define_length_type!(LengthBoth, LengthDir::Both);
+
+#[derive(Debug, PartialEq, Copy, Clone)]
+struct Length {
+    pub length: f64,
+    pub unit: LengthUnit,
+}
+
 pub const POINTS_PER_INCH: f64 = 72.0;
 const CM_PER_INCH: f64 = 2.54;
 const MM_PER_INCH: f64 = 25.4;
@@ -62,11 +164,10 @@ fn make_err() -> ValueErrorKind {
 }
 
 impl Parse for Length {
-    type Data = LengthDir;
     type Err = ValueErrorKind;
 
-    fn parse(parser: &mut Parser<'_, '_>, dir: LengthDir) -> Result<Length, ValueErrorKind> {
-        let length = Length::from_cssparser(parser, dir)?;
+    fn parse(parser: &mut Parser<'_, '_>) -> Result<Length, ValueErrorKind> {
+        let length = Length::from_cssparser(parser)?;
 
         parser.expect_exhausted().map_err(|_| make_err())?;
 
@@ -75,43 +176,8 @@ impl Parse for Length {
 }
 
 impl Length {
-    pub fn new(l: f64, unit: LengthUnit, dir: LengthDir) -> Length {
-        Length {
-            length: l,
-            unit,
-            dir,
-        }
-    }
-
-    pub fn check_nonnegative(self) -> Result<Length, ValueErrorKind> {
-        if self.length >= 0.0 {
-            Ok(self)
-        } else {
-            Err(ValueErrorKind::Value(
-                "value must be non-negative".to_string(),
-            ))
-        }
-    }
-
-    pub fn normalize(&self, values: &ComputedValues, params: &ViewParams) -> f64 {
-        match self.unit {
-            LengthUnit::Default => self.length,
-
-            LengthUnit::Percent => match self.dir {
-                LengthDir::Horizontal => self.length * params.view_box_width(),
-                LengthDir::Vertical => self.length * params.view_box_height(),
-                LengthDir::Both => {
-                    self.length
-                        * viewport_percentage(params.view_box_width(), params.view_box_height())
-                }
-            },
-
-            LengthUnit::FontEm => self.length * font_size_from_values(values, params),
-
-            LengthUnit::FontEx => self.length * font_size_from_values(values, params) / 2.0,
-
-            LengthUnit::Inch => font_size_from_inch(self.length, self.dir, params),
-        }
+    fn new(l: f64, unit: LengthUnit) -> Length {
+        Length { length: l, unit }
     }
 
     /// Returns the raw length after asserting units are either default or percent.
@@ -136,10 +202,7 @@ impl Length {
         }
     }
 
-    pub fn from_cssparser(
-        parser: &mut Parser<'_, '_>,
-        dir: LengthDir,
-    ) -> Result<Length, ValueErrorKind> {
+    pub fn from_cssparser(parser: &mut Parser<'_, '_>) -> Result<Length, ValueErrorKind> {
         let length = {
             let token = parser.next().map_err(|_| {
                 ValueErrorKind::Parse(ParseError::new(
@@ -149,69 +212,59 @@ impl Length {
 
             match *token {
                 Token::Number { value, .. } => Length {
-                    length: f64::from(finite_f32(value)?),
+                    length: f64::from(value),
                     unit: LengthUnit::Default,
-                    dir,
                 },
 
                 Token::Percentage { unit_value, .. } => Length {
-                    length: f64::from(finite_f32(unit_value)?),
+                    length: f64::from(unit_value),
                     unit: LengthUnit::Percent,
-                    dir,
                 },
 
                 Token::Dimension {
                     value, ref unit, ..
                 } => {
-                    let value = f64::from(finite_f32(value)?);
+                    let value = f64::from(value);
 
                     match unit.as_ref() {
                         "em" => Length {
                             length: value,
                             unit: LengthUnit::FontEm,
-                            dir,
                         },
 
                         "ex" => Length {
                             length: value,
                             unit: LengthUnit::FontEx,
-                            dir,
                         },
 
                         "pt" => Length {
                             length: value / POINTS_PER_INCH,
                             unit: LengthUnit::Inch,
-                            dir,
                         },
 
                         "in" => Length {
                             length: value,
                             unit: LengthUnit::Inch,
-                            dir,
                         },
 
                         "cm" => Length {
                             length: value / CM_PER_INCH,
                             unit: LengthUnit::Inch,
-                            dir,
                         },
 
                         "mm" => Length {
                             length: value / MM_PER_INCH,
                             unit: LengthUnit::Inch,
-                            dir,
                         },
 
                         "pc" => Length {
                             length: value / PICA_PER_INCH,
                             unit: LengthUnit::Inch,
-                            dir,
                         },
 
                         "px" => Length {
                             length: value,
                             unit: LengthUnit::Default,
-                            dir,
                         },
 
                         _ => return Err(make_err()),
@@ -226,21 +279,14 @@ impl Length {
     }
 }
 
-fn font_size_from_inch(length: f64, dir: LengthDir, params: &ViewParams) -> f64 {
-    match dir {
-        LengthDir::Horizontal => length * params.dpi_x(),
-        LengthDir::Vertical => length * params.dpi_y(),
-        LengthDir::Both => length * viewport_percentage(params.dpi_x(), params.dpi_y()),
-    }
-}
-
 fn font_size_from_values(values: &ComputedValues, params: &ViewParams) -> f64 {
-    let v = &values.font_size.0.value();
+    let v = &values.font_size.0.value().0;
 
     match v.unit {
         LengthUnit::Default => v.length,
 
-        LengthUnit::Inch => font_size_from_inch(v.length, v.dir, params),
+        // FontSize always is a LengthDir::Both, per properties.rs
+        LengthUnit::Inch => v.length * LengthDir::Both.scaling_factor(params.dpi_x, params.dpi_y),
 
         LengthUnit::Percent => unreachable!("ComputedValues can't have a relative font size"),
 
@@ -262,7 +308,7 @@ fn viewport_percentage(x: f64, y: f64) -> f64 {
 #[derive(Debug, PartialEq, Clone)]
 pub enum Dasharray {
     None,
-    Array(Vec<Length>),
+    Array(Vec<LengthBoth>),
 }
 
 impl Default for Dasharray {
@@ -272,10 +318,9 @@ impl Default for Dasharray {
 }
 
 impl Parse for Dasharray {
-    type Data = ();
     type Err = ValueErrorKind;
 
-    fn parse(parser: &mut Parser<'_, '_>, _: Self::Data) -> Result<Dasharray, ValueErrorKind> {
+    fn parse(parser: &mut Parser<'_, '_>) -> Result<Dasharray, ValueErrorKind> {
         if parser.try(|p| p.expect_ident_matching("none")).is_ok() {
             Ok(Dasharray::None)
         } else {
@@ -285,13 +330,11 @@ impl Parse for Dasharray {
 }
 
 // This does not handle "inherit" or "none" state, the caller is responsible for that.
-fn parse_dash_array(parser: &mut Parser<'_, '_>) -> Result<Vec<Length>, ValueErrorKind> {
+fn parse_dash_array(parser: &mut Parser<'_, '_>) -> Result<Vec<LengthBoth>, ValueErrorKind> {
     let mut dasharray = Vec::new();
 
     loop {
-        dasharray.push(
-            Length::from_cssparser(parser, LengthDir::Both).and_then(Length::check_nonnegative)?,
-        );
+        dasharray.push(LengthBoth::from_cssparser(parser).and_then(LengthBoth::check_nonnegative)?);
 
         if parser.is_exhausted() {
             break;
@@ -312,95 +355,84 @@ mod tests {
     #[test]
     fn parses_default() {
         assert_eq!(
-            Length::parse_str("42", LengthDir::Horizontal),
-            Ok(Length::new(
-                42.0,
-                LengthUnit::Default,
-                LengthDir::Horizontal
-            ))
+            LengthHorizontal::parse_str("42"),
+            Ok(LengthHorizontal(Length::new(42.0, LengthUnit::Default,)))
         );
 
         assert_eq!(
-            Length::parse_str("-42px", LengthDir::Horizontal),
-            Ok(Length::new(
-                -42.0,
-                LengthUnit::Default,
-                LengthDir::Horizontal
-            ))
+            LengthHorizontal::parse_str("-42px"),
+            Ok(LengthHorizontal(Length::new(-42.0, LengthUnit::Default,)))
         );
     }
 
     #[test]
     fn parses_percent() {
         assert_eq!(
-            Length::parse_str("50.0%", LengthDir::Horizontal),
-            Ok(Length::new(0.5, LengthUnit::Percent, LengthDir::Horizontal))
+            LengthHorizontal::parse_str("50.0%"),
+            Ok(LengthHorizontal(Length::new(0.5, LengthUnit::Percent,)))
         );
     }
 
     #[test]
     fn parses_font_em() {
         assert_eq!(
-            Length::parse_str("22.5em", LengthDir::Vertical),
-            Ok(Length::new(22.5, LengthUnit::FontEm, LengthDir::Vertical))
+            LengthVertical::parse_str("22.5em"),
+            Ok(LengthVertical(Length::new(22.5, LengthUnit::FontEm,)))
         );
     }
 
     #[test]
     fn parses_font_ex() {
         assert_eq!(
-            Length::parse_str("22.5ex", LengthDir::Vertical),
-            Ok(Length::new(22.5, LengthUnit::FontEx, LengthDir::Vertical))
+            LengthVertical::parse_str("22.5ex"),
+            Ok(LengthVertical(Length::new(22.5, LengthUnit::FontEx,)))
         );
     }
 
     #[test]
     fn parses_physical_units() {
         assert_eq!(
-            Length::parse_str("72pt", LengthDir::Both),
-            Ok(Length::new(1.0, LengthUnit::Inch, LengthDir::Both))
+            LengthBoth::parse_str("72pt"),
+            Ok(LengthBoth(Length::new(1.0, LengthUnit::Inch,)))
         );
 
         assert_eq!(
-            Length::parse_str("-22.5in", LengthDir::Both),
-            Ok(Length::new(-22.5, LengthUnit::Inch, LengthDir::Both))
+            LengthBoth::parse_str("-22.5in"),
+            Ok(LengthBoth(Length::new(-22.5, LengthUnit::Inch,)))
         );
 
         assert_eq!(
-            Length::parse_str("-254cm", LengthDir::Both),
-            Ok(Length::new(-100.0, LengthUnit::Inch, LengthDir::Both))
+            LengthBoth::parse_str("-254cm"),
+            Ok(LengthBoth(Length::new(-100.0, LengthUnit::Inch,)))
         );
 
         assert_eq!(
-            Length::parse_str("254mm", LengthDir::Both),
-            Ok(Length::new(10.0, LengthUnit::Inch, LengthDir::Both))
+            LengthBoth::parse_str("254mm"),
+            Ok(LengthBoth(Length::new(10.0, LengthUnit::Inch,)))
         );
 
         assert_eq!(
-            Length::parse_str("60pc", LengthDir::Both),
-            Ok(Length::new(10.0, LengthUnit::Inch, LengthDir::Both))
+            LengthBoth::parse_str("60pc"),
+            Ok(LengthBoth(Length::new(10.0, LengthUnit::Inch,)))
         );
     }
 
     #[test]
     fn empty_length_yields_error() {
-        assert!(is_parse_error(&Length::parse_str("", LengthDir::Both)));
+        assert!(is_parse_error(&LengthBoth::parse_str("")));
     }
 
     #[test]
     fn invalid_unit_yields_error() {
-        assert!(is_parse_error(&Length::parse_str(
-            "8furlong",
-            LengthDir::Both
-        )));
+        assert!(is_parse_error(&LengthBoth::parse_str("8furlong")));
     }
 
     #[test]
     fn check_nonnegative_works() {
-        assert!(Length::parse_str("0", LengthDir::Both)
+        assert!(LengthBoth::parse_str("0")
             .and_then(|l| l.check_nonnegative())
             .is_ok());
-        assert!(Length::parse_str("-10", LengthDir::Both)
+        assert!(LengthBoth::parse_str("-10")
             .and_then(|l| l.check_nonnegative())
             .is_err());
     }
@@ -412,7 +444,7 @@ mod tests {
         let values = ComputedValues::default();
 
         assert_approx_eq_cairo!(
-            Length::new(10.0, LengthUnit::Default, LengthDir::Both).normalize(&values, &params),
+            LengthBoth::new(10.0, LengthUnit::Default).normalize(&values, &params),
             10.0
         );
     }
@@ -424,11 +456,11 @@ mod tests {
         let values = ComputedValues::default();
 
         assert_approx_eq_cairo!(
-            Length::new(10.0, LengthUnit::Inch, LengthDir::Horizontal).normalize(&values, &params),
+            LengthHorizontal::new(10.0, LengthUnit::Inch).normalize(&values, &params),
             400.0
         );
         assert_approx_eq_cairo!(
-            Length::new(10.0, LengthUnit::Inch, LengthDir::Vertical).normalize(&values, &params),
+            LengthVertical::new(10.0, LengthUnit::Inch).normalize(&values, &params),
             500.0
         );
     }
@@ -440,12 +472,11 @@ mod tests {
         let values = ComputedValues::default();
 
         assert_approx_eq_cairo!(
-            Length::new(0.05, LengthUnit::Percent, LengthDir::Horizontal)
-                .normalize(&values, &params),
+            LengthHorizontal::new(0.05, LengthUnit::Percent).normalize(&values, &params),
             5.0
         );
         assert_approx_eq_cairo!(
-            Length::new(0.05, LengthUnit::Percent, LengthDir::Vertical).normalize(&values, &params),
+            LengthVertical::new(0.05, LengthUnit::Percent).normalize(&values, &params),
             10.0
         );
     }
@@ -460,24 +491,24 @@ mod tests {
         // property and the way we compute FontEx from that.
 
         assert_approx_eq_cairo!(
-            Length::new(1.0, LengthUnit::FontEm, LengthDir::Vertical).normalize(&values, &params),
+            LengthVertical::new(1.0, LengthUnit::FontEm).normalize(&values, &params),
             12.0
         );
 
         assert_approx_eq_cairo!(
-            Length::new(1.0, LengthUnit::FontEx, LengthDir::Vertical).normalize(&values, &params),
+            LengthVertical::new(1.0, LengthUnit::FontEx).normalize(&values, &params),
             6.0
         );
     }
 
     fn parse_dash_array_str(s: &str) -> Result<Dasharray, ValueErrorKind> {
-        Dasharray::parse_str(s, ())
+        Dasharray::parse_str(s)
     }
 
     #[test]
     fn parses_dash_array() {
         // helper to cut down boilderplate
-        let length_parse = |s| Length::parse_str(s, LengthDir::Both).unwrap();
+        let length_parse = |s| LengthBoth::parse_str(s).unwrap();
 
         let expected = Dasharray::Array(vec![
             length_parse("1"),
