@@ -35,6 +35,11 @@ extern crate xml as xml_rs;
 #[macro_use]
 extern crate lazy_static;
 
+pub use c_api::{
+    rsvg_rust_error_get_type,
+    rsvg_rust_handle_flags_get_type,
+};
+
 pub use color::{rsvg_css_parse_color, ColorKind, ColorSpec};
 
 pub use dpi::rsvg_rust_set_default_dpi_x_y;
@@ -54,7 +59,11 @@ pub use handle::{
     rsvg_handle_rust_get_position_sub,
     rsvg_handle_rust_has_sub,
     rsvg_handle_rust_new,
+    rsvg_handle_rust_new_from_data,
+    rsvg_handle_rust_new_from_file,
+    rsvg_handle_rust_new_from_gfile_sync,
     rsvg_handle_rust_new_from_stream_sync,
+    rsvg_handle_rust_new_with_flags,
     rsvg_handle_rust_read_stream_sync,
     rsvg_handle_rust_render_cairo_sub,
     rsvg_handle_rust_set_base_gfile,
@@ -67,6 +76,13 @@ pub use handle::{
     rsvg_handle_rust_write,
 };
 
+pub use pixbuf_utils::{
+    rsvg_rust_pixbuf_from_file_at_max_size,
+    rsvg_rust_pixbuf_from_file_at_size,
+    rsvg_rust_pixbuf_from_file_at_zoom,
+    rsvg_rust_pixbuf_from_file_at_zoom_with_max,
+};
+
 pub use xml::rsvg_xml_state_error;
 
 #[macro_use]
@@ -86,13 +102,13 @@ mod angle;
 mod aspect_ratio;
 mod attributes;
 mod bbox;
+mod c_api;
 mod clip_path;
 mod color;
 mod cond;
 mod create_node;
 mod croco;
 mod css;
-mod defs;
 mod dpi;
 mod drawing_ctx;
 mod error;
@@ -105,21 +121,22 @@ mod io;
 mod iri;
 mod length;
 mod link;
-mod load;
 mod marker;
 mod mask;
 mod node;
+mod number_list;
 mod paint_server;
 mod parsers;
 mod path_builder;
 mod path_parser;
 mod pattern;
+mod pixbuf_utils;
+mod properties;
 mod property_bag;
 mod rect;
 mod shapes;
 mod space;
 pub mod srgb;
-mod state;
 mod stop;
 mod structure;
 mod style;
@@ -127,7 +144,6 @@ pub mod surface_utils;
 mod svg;
 mod text;
 mod transform;
-mod tree;
 pub mod tree_utils;
 mod unit_interval;
 mod util;
