@@ -5,20 +5,20 @@ use std::ops::Deref;
 use cairo::MatrixTrait;
 use cssparser::{CowRcStr, Parser, Token};
 
+use allowed_url::Fragment;
 use angle::Angle;
 use aspect_ratio::*;
 use attributes::Attribute;
-use defs::Fragment;
 use drawing_ctx::DrawingCtx;
 use error::*;
 use float_eq_cairo::ApproxEqCairo;
 use iri::IRI;
-use length::{Length, LengthDir};
+use length::{LengthHorizontal, LengthVertical};
 use node::*;
 use parsers::{Parse, ParseError, ParseValue};
 use path_builder::*;
+use properties::{ComputedValues, SpecifiedValue, SpecifiedValues};
 use property_bag::PropertyBag;
-use state::{ComputedValues, SpecifiedValue, State};
 use viewbox::*;
 
 // markerUnits attribute: https://www.w3.org/TR/SVG/painting.html#MarkerElement
@@ -35,10 +35,9 @@ impl Default for MarkerUnits {
 }
 
 impl Parse for MarkerUnits {
-    type Data = ();
     type Err = ValueErrorKind;
 
-    fn parse(parser: &mut Parser<'_, '_>, _: ()) -> Result<MarkerUnits, ValueErrorKind> {
+    fn parse(parser: &mut Parser<'_, '_>) -> Result<MarkerUnits, ValueErrorKind> {
         let loc = parser.current_source_location();
 
         parser
@@ -74,24 +73,23 @@ impl Default for MarkerOrient {
 }
 
 impl Parse for MarkerOrient {
-    type Data = ();
     type Err = ValueErrorKind;
 
-    fn parse(parser: &mut Parser<'_, '_>, _: ()) -> Result<MarkerOrient, ValueErrorKind> {
+    fn parse(parser: &mut Parser<'_, '_>) -> Result<MarkerOrient, ValueErrorKind> {
         if parser.try(|p| p.expect_ident_matching("auto")).is_ok() {
             Ok(MarkerOrient::Auto)
         } else {
-            Angle::parse(parser, ()).map(MarkerOrient::Angle)
+            Angle::parse(parser).map(MarkerOrient::Angle)
         }
     }
 }
 
 pub struct NodeMarker {
     units: Cell<MarkerUnits>,
-    ref_x: Cell<Length>,
-    ref_y: Cell<Length>,
-    width: Cell<Length>,
-    height: Cell<Length>,
+    ref_x: Cell<LengthHorizontal>,
+    ref_y: Cell<LengthVertical>,
+    width: Cell<LengthHorizontal>,
+    height: Cell<LengthVertical>,
     orient: Cell<MarkerOrient>,
     aspect: Cell<AspectRatio>,
     vbox: Cell<Option<ViewBox>>,
@@ -101,21 +99,17 @@ impl NodeMarker {
     pub fn new() -> NodeMarker {
         NodeMarker {
             units: Cell::new(MarkerUnits::default()),
-            ref_x: Cell::new(Length::default()),
-            ref_y: Cell::new(Length::default()),
-            width: Cell::new(NodeMarker::get_default_size(LengthDir::Horizontal)),
-            height: Cell::new(NodeMarker::get_default_size(LengthDir::Vertical)),
+            ref_x: Cell::new(Default::default()),
+            ref_y: Cell::new(Default::default()),
+            // the following two are per the spec
+            width: Cell::new(LengthHorizontal::parse_str("3").unwrap()),
+            height: Cell::new(LengthVertical::parse_str("3").unwrap()),
             orient: Cell::new(MarkerOrient::default()),
             aspect: Cell::new(AspectRatio::default()),
             vbox: Cell::new(None),
         }
     }
 
-    fn get_default_size(dir: LengthDir) -> Length {
-        // per the spec
-        Length::parse_str("3", dir).unwrap()
-    }
-
     fn render(
         &self,
         node: &RsvgNode,
@@ -159,21 +153,21 @@ impl NodeMarker {
 
         let params = if let Some(vbox) = self.vbox.get() {
             let (_, _, w, h) = self.aspect.get().compute(
-                vbox.0.width,
-                vbox.0.height,
+                vbox.width,
+                vbox.height,
                 0.0,
                 0.0,
                 marker_width,
                 marker_height,
             );
 
-            if vbox.0.width.approx_eq_cairo(&0.0) || vbox.0.height.approx_eq_cairo(&0.0) {
+            if vbox.width.approx_eq_cairo(&0.0) || vbox.height.approx_eq_cairo(&0.0) {
                 return Ok(());
             }
 
-            affine.scale(w / vbox.0.width, h / vbox.0.height);
+            affine.scale(w / vbox.width, h / vbox.height);
 
-            draw_ctx.push_view_box(vbox.0.width, vbox.0.height)
+            draw_ctx.push_view_box(vbox.width, vbox.height)
         } else {
             draw_ctx.push_view_box(marker_width, marker_height)
         };
@@ -189,7 +183,7 @@ impl NodeMarker {
 
         if !values.is_overflow() {
             if let Some(vbox) = self.vbox.get() {
-                draw_ctx.clip(vbox.0.x, vbox.0.y, vbox.0.width, vbox.0.height);
+                draw_ctx.clip(vbox.x, vbox.y, vbox.width, vbox.height);
             } else {
                 draw_ctx.clip(0.0, 0.0, marker_width, marker_height);
             }
@@ -213,29 +207,25 @@ impl NodeTrait for NodeMarker {
 
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::MarkerUnits => self.units.set(attr.parse(value, ())?),
+                Attribute::MarkerUnits => self.units.set(attr.parse(value)?),
 
-                Attribute::RefX => self.ref_x.set(attr.parse(value, LengthDir::Horizontal)?),
+                Attribute::RefX => self.ref_x.set(attr.parse(value)?),
 
-                Attribute::RefY => self.ref_y.set(attr.parse(value, LengthDir::Vertical)?),
+                Attribute::RefY => self.ref_y.set(attr.parse(value)?),
 
-                Attribute::MarkerWidth => self.width.set(attr.parse_and_validate(
-                    value,
-                    LengthDir::Horizontal,
-                    Length::check_nonnegative,
-                )?),
+                Attribute::MarkerWidth => self
+                    .width
+                    .set(attr.parse_and_validate(value, LengthHorizontal::check_nonnegative)?),
 
-                Attribute::MarkerHeight => self.height.set(attr.parse_and_validate(
-                    value,
-                    LengthDir::Vertical,
-                    Length::check_nonnegative,
-                )?),
+                Attribute::MarkerHeight => self
+                    .height
+                    .set(attr.parse_and_validate(value, LengthVertical::check_nonnegative)?),
 
-                Attribute::Orient => self.orient.set(attr.parse(value, ())?),
+                Attribute::Orient => self.orient.set(attr.parse(value)?),
 
-                Attribute::PreserveAspectRatio => self.aspect.set(attr.parse(value, ())?),
+                Attribute::PreserveAspectRatio => self.aspect.set(attr.parse(value)?),
 
-                Attribute::ViewBox => self.vbox.set(Some(attr.parse(value, ())?)),
+                Attribute::ViewBox => self.vbox.set(Some(attr.parse(value)?)),
 
                 _ => (),
             }
@@ -244,9 +234,9 @@ impl NodeTrait for NodeMarker {
         Ok(())
     }
 
-    fn set_overridden_properties(&self, state: &mut State) {
+    fn set_overridden_properties(&self, values: &mut SpecifiedValues) {
         // markers are always displayed, even if <marker> or its ancestors are display:none
-        state.values.display = SpecifiedValue::Specified(Default::default());
+        values.display = SpecifiedValue::Specified(Default::default());
     }
 }
 
@@ -837,21 +827,21 @@ mod parser_tests {
     #[test]
     fn parsing_invalid_marker_units_yields_error() {
         assert!(is_parse_error(
-            &MarkerUnits::parse_str("", ()).map_err(|e| ValueErrorKind::from(e))
+            &MarkerUnits::parse_str("").map_err(|e| ValueErrorKind::from(e))
         ));
         assert!(is_parse_error(
-            &MarkerUnits::parse_str("foo", ()).map_err(|e| ValueErrorKind::from(e))
+            &MarkerUnits::parse_str("foo").map_err(|e| ValueErrorKind::from(e))
         ));
     }
 
     #[test]
     fn parses_marker_units() {
         assert_eq!(
-            MarkerUnits::parse_str("userSpaceOnUse", ()),
+            MarkerUnits::parse_str("userSpaceOnUse"),
             Ok(MarkerUnits::UserSpaceOnUse)
         );
         assert_eq!(
-            MarkerUnits::parse_str("strokeWidth", ()),
+            MarkerUnits::parse_str("strokeWidth"),
             Ok(MarkerUnits::StrokeWidth)
         );
     }
@@ -859,38 +849,38 @@ mod parser_tests {
     #[test]
     fn parsing_invalid_marker_orient_yields_error() {
         assert!(is_parse_error(
-            &MarkerOrient::parse_str("", ()).map_err(|e| ValueErrorKind::from(e))
+            &MarkerOrient::parse_str("").map_err(|e| ValueErrorKind::from(e))
         ));
         assert!(is_parse_error(
-            &MarkerOrient::parse_str("blah", ()).map_err(|e| ValueErrorKind::from(e))
+            &MarkerOrient::parse_str("blah").map_err(|e| ValueErrorKind::from(e))
         ));
         assert!(is_parse_error(
-            &MarkerOrient::parse_str("45blah", ()).map_err(|e| ValueErrorKind::from(e))
+            &MarkerOrient::parse_str("45blah").map_err(|e| ValueErrorKind::from(e))
         ));
     }
 
     #[test]
     fn parses_marker_orient() {
-        assert_eq!(MarkerOrient::parse_str("auto", ()), Ok(MarkerOrient::Auto));
+        assert_eq!(MarkerOrient::parse_str("auto"), Ok(MarkerOrient::Auto));
 
         assert_eq!(
-            MarkerOrient::parse_str("0", ()),
+            MarkerOrient::parse_str("0"),
             Ok(MarkerOrient::Angle(Angle::new(0.0)))
         );
         assert_eq!(
-            MarkerOrient::parse_str("180", ()),
+            MarkerOrient::parse_str("180"),
             Ok(MarkerOrient::Angle(Angle::from_degrees(180.0)))
         );
         assert_eq!(
-            MarkerOrient::parse_str("180deg", ()),
+            MarkerOrient::parse_str("180deg"),
             Ok(MarkerOrient::Angle(Angle::from_degrees(180.0)))
         );
         assert_eq!(
-            MarkerOrient::parse_str("-400grad", ()),
+            MarkerOrient::parse_str("-400grad"),
             Ok(MarkerOrient::Angle(Angle::from_degrees(-360.0)))
         );
         assert_eq!(
-            MarkerOrient::parse_str("1rad", ()),
+            MarkerOrient::parse_str("1rad"),
             Ok(MarkerOrient::Angle(Angle::new(1.0)))
         );
     }
