@@ -5,11 +5,11 @@ use std::cell::RefCell;
 use std::f64;
 use std::rc::*;
 
+use allowed_url::Fragment;
 use aspect_ratio::*;
 use attributes::Attribute;
 use bbox::*;
 use coord_units::CoordUnits;
-use defs::Fragment;
 use drawing_ctx::{DrawingCtx, NodeStack};
 use error::{AttributeResultExt, RenderingError};
 use float_eq_cairo::ApproxEqCairo;
@@ -17,8 +17,8 @@ use length::*;
 use node::*;
 use paint_server::PaintSource;
 use parsers::ParseValue;
+use properties::ComputedValues;
 use property_bag::PropertyBag;
-use state::ComputedValues;
 use unit_interval::UnitInterval;
 use viewbox::*;
 
@@ -37,10 +37,10 @@ pub struct Pattern {
     pub preserve_aspect_ratio: Option<AspectRatio>,
     pub affine: Option<cairo::Matrix>,
     pub fallback: Option<Fragment>,
-    pub x: Option<Length>,
-    pub y: Option<Length>,
-    pub width: Option<Length>,
-    pub height: Option<Length>,
+    pub x: Option<LengthHorizontal>,
+    pub y: Option<LengthVertical>,
+    pub width: Option<LengthHorizontal>,
+    pub height: Option<LengthVertical>,
 
     // Point back to our corresponding node, or to the fallback node which has children.
     // If the value is None, it means we are fully resolved and didn't find any children
@@ -59,10 +59,10 @@ impl Default for Pattern {
             preserve_aspect_ratio: Some(AspectRatio::default()),
             affine: Some(cairo::Matrix::identity()),
             fallback: None,
-            x: Some(Length::default()),
-            y: Some(Length::default()),
-            width: Some(Length::default()),
-            height: Some(Length::default()),
+            x: Some(Default::default()),
+            y: Some(Default::default()),
+            width: Some(Default::default()),
+            height: Some(Default::default()),
             node: None,
         }
     }
@@ -183,40 +183,34 @@ impl NodeTrait for NodePattern {
 
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::PatternUnits => p.units = Some(attr.parse(value, ())?),
+                Attribute::PatternUnits => p.units = Some(attr.parse(value)?),
 
-                Attribute::PatternContentUnits => p.content_units = Some(attr.parse(value, ())?),
+                Attribute::PatternContentUnits => p.content_units = Some(attr.parse(value)?),
 
-                Attribute::ViewBox => p.vbox = Some(Some(attr.parse(value, ())?)),
+                Attribute::ViewBox => p.vbox = Some(Some(attr.parse(value)?)),
 
                 Attribute::PreserveAspectRatio => {
-                    p.preserve_aspect_ratio = Some(attr.parse(value, ())?)
+                    p.preserve_aspect_ratio = Some(attr.parse(value)?)
                 }
 
-                Attribute::PatternTransform => p.affine = Some(attr.parse(value, ())?),
+                Attribute::PatternTransform => p.affine = Some(attr.parse(value)?),
 
                 Attribute::XlinkHref => {
                     p.fallback = Some(Fragment::parse(value).attribute(Attribute::XlinkHref)?);
                 }
 
-                Attribute::X => p.x = Some(attr.parse(value, LengthDir::Horizontal)?),
+                Attribute::X => p.x = Some(attr.parse(value)?),
 
-                Attribute::Y => p.y = Some(attr.parse(value, LengthDir::Vertical)?),
+                Attribute::Y => p.y = Some(attr.parse(value)?),
 
                 Attribute::Width => {
-                    p.width = Some(attr.parse_and_validate(
-                        value,
-                        LengthDir::Horizontal,
-                        Length::check_nonnegative,
-                    )?)
+                    p.width =
+                        Some(attr.parse_and_validate(value, LengthHorizontal::check_nonnegative)?)
                 }
 
                 Attribute::Height => {
-                    p.height = Some(attr.parse_and_validate(
-                        value,
-                        LengthDir::Vertical,
-                        Length::check_nonnegative,
-                    )?)
+                    p.height =
+                        Some(attr.parse_and_validate(value, LengthVertical::check_nonnegative)?)
                 }
 
                 _ => (),
@@ -373,20 +367,20 @@ impl PaintSource for NodePattern {
         let _params = if let Some(vbox) = vbox {
             // If there is a vbox, use that
             let (mut x, mut y, w, h) = preserve_aspect_ratio.compute(
-                vbox.0.width,
-                vbox.0.height,
+                vbox.width,
+                vbox.height,
                 0.0,
                 0.0,
                 pattern_width * bbwscale,
                 pattern_height * bbhscale,
             );
 
-            x -= vbox.0.x * w / vbox.0.width;
-            y -= vbox.0.y * h / vbox.0.height;
+            x -= vbox.x * w / vbox.width;
+            y -= vbox.y * h / vbox.height;
 
-            caffine = cairo::Matrix::new(w / vbox.0.width, 0.0, 0.0, h / vbox.0.height, x, y);
+            caffine = cairo::Matrix::new(w / vbox.width, 0.0, 0.0, h / vbox.height, x, y);
 
-            draw_ctx.push_view_box(vbox.0.width, vbox.0.height)
+            draw_ctx.push_view_box(vbox.width, vbox.height)
         } else if content_units == PatternContentUnits(CoordUnits::ObjectBoundingBox) {
             // If coords are in terms of the bounding box, use them
             let bbrect = bbox.rect.unwrap();
