@@ -1,10 +1,10 @@
 use std::cell::Cell;
 use std::cell::RefCell;
 
+use allowed_url::Fragment;
 use aspect_ratio::*;
 use attributes::Attribute;
 use css::CssStyles;
-use defs::Fragment;
 use dpi::Dpi;
 use drawing_ctx::DrawingCtx;
 use error::{AttributeResultExt, RenderingError};
@@ -12,8 +12,8 @@ use float_eq_cairo::ApproxEqCairo;
 use length::*;
 use node::*;
 use parsers::{Parse, ParseValue};
+use properties::Overflow;
 use property_bag::{OwnedPropertyBag, PropertyBag};
-use state::Overflow;
 use viewbox::*;
 use viewport::{draw_in_viewport, ClipMode};
 
@@ -97,10 +97,10 @@ impl NodeTrait for NodeSwitch {
 
 pub struct NodeSvg {
     preserve_aspect_ratio: Cell<AspectRatio>,
-    x: Cell<Length>,
-    y: Cell<Length>,
-    w: Cell<Length>,
-    h: Cell<Length>,
+    x: Cell<LengthHorizontal>,
+    y: Cell<LengthVertical>,
+    w: Cell<LengthHorizontal>,
+    h: Cell<LengthVertical>,
     vbox: Cell<Option<ViewBox>>,
     pbag: RefCell<Option<OwnedPropertyBag>>,
 }
@@ -109,10 +109,10 @@ impl NodeSvg {
     pub fn new() -> NodeSvg {
         NodeSvg {
             preserve_aspect_ratio: Cell::new(AspectRatio::default()),
-            x: Cell::new(Length::parse_str("0", LengthDir::Horizontal).unwrap()),
-            y: Cell::new(Length::parse_str("0", LengthDir::Vertical).unwrap()),
-            w: Cell::new(Length::parse_str("100%", LengthDir::Horizontal).unwrap()),
-            h: Cell::new(Length::parse_str("100%", LengthDir::Vertical).unwrap()),
+            x: Cell::new(LengthHorizontal::parse_str("0").unwrap()),
+            y: Cell::new(LengthVertical::parse_str("0").unwrap()),
+            w: Cell::new(LengthHorizontal::parse_str("100%").unwrap()),
+            h: Cell::new(LengthVertical::parse_str("100%").unwrap()),
             vbox: Cell::new(None),
             pbag: RefCell::new(None),
         }
@@ -125,19 +125,16 @@ impl NodeSvg {
         }
     }
 
-    pub fn get_size(&self, dpi: &Dpi) -> Option<(i32, i32)> {
-        let dpi_x = dpi.x();
-        let dpi_y = dpi.y();
-
+    pub fn get_size(&self, dpi: Dpi) -> Option<(i32, i32)> {
         match (self.w.get(), self.h.get(), self.vbox.get()) {
-            (w, h, Some(vb)) => Some((
-                w.hand_normalize(dpi_x, vb.0.width, 12.0).round() as i32,
-                h.hand_normalize(dpi_y, vb.0.height, 12.0).round() as i32,
+            (w, h, Some(vbox)) => Some((
+                w.hand_normalize(dpi.x(), vbox.width, 12.0).round() as i32,
+                h.hand_normalize(dpi.y(), vbox.height, 12.0).round() as i32,
             )),
-            (w, h, None) if w.unit != LengthUnit::Percent && h.unit != LengthUnit::Percent => {
+            (w, h, None) if w.unit() != LengthUnit::Percent && h.unit() != LengthUnit::Percent => {
                 Some((
-                    w.hand_normalize(dpi_x, 0.0, 12.0).round() as i32,
-                    h.hand_normalize(dpi_y, 0.0, 12.0).round() as i32,
+                    w.hand_normalize(dpi.x(), 0.0, 12.0).round() as i32,
+                    h.hand_normalize(dpi.y(), 0.0, 12.0).round() as i32,
                 ))
             }
             (_, _, _) => None,
@@ -158,34 +155,30 @@ impl NodeTrait for NodeSvg {
         for (attr, value) in pbag.iter() {
             match attr {
                 Attribute::PreserveAspectRatio => {
-                    self.preserve_aspect_ratio.set(attr.parse(value, ())?)
+                    self.preserve_aspect_ratio.set(attr.parse(value)?)
                 }
 
                 Attribute::X => {
                     if is_inner_svg {
-                        self.x.set(attr.parse(value, LengthDir::Horizontal)?);
+                        self.x.set(attr.parse(value)?);
                     }
                 }
 
                 Attribute::Y => {
                     if is_inner_svg {
-                        self.y.set(attr.parse(value, LengthDir::Vertical)?);
+                        self.y.set(attr.parse(value)?);
                     }
                 }
 
-                Attribute::Width => self.w.set(attr.parse_and_validate(
-                    value,
-                    LengthDir::Horizontal,
-                    Length::check_nonnegative,
-                )?),
+                Attribute::Width => self
+                    .w
+                    .set(attr.parse_and_validate(value, LengthHorizontal::check_nonnegative)?),
 
-                Attribute::Height => self.h.set(attr.parse_and_validate(
-                    value,
-                    LengthDir::Vertical,
-                    Length::check_nonnegative,
-                )?),
+                Attribute::Height => self
+                    .h
+                    .set(attr.parse_and_validate(value, LengthVertical::check_nonnegative)?),
 
-                Attribute::ViewBox => self.vbox.set(attr.parse(value, ()).map(Some)?),
+                Attribute::ViewBox => self.vbox.set(attr.parse(value).map(Some)?),
 
                 _ => (),
             }
@@ -240,18 +233,18 @@ impl NodeTrait for NodeSvg {
 
 pub struct NodeUse {
     link: RefCell<Option<Fragment>>,
-    x: Cell<Length>,
-    y: Cell<Length>,
-    w: Cell<Option<Length>>,
-    h: Cell<Option<Length>>,
+    x: Cell<LengthHorizontal>,
+    y: Cell<LengthVertical>,
+    w: Cell<Option<LengthHorizontal>>,
+    h: Cell<Option<LengthVertical>>,
 }
 
 impl NodeUse {
     pub fn new() -> NodeUse {
         NodeUse {
             link: RefCell::new(None),
-            x: Cell::new(Length::default()),
-            y: Cell::new(Length::default()),
+            x: Cell::new(Default::default()),
+            y: Cell::new(Default::default()),
             w: Cell::new(None),
             h: Cell::new(None),
         }
@@ -267,19 +260,15 @@ impl NodeTrait for NodeUse {
                         Some(Fragment::parse(value).attribute(Attribute::XlinkHref)?)
                 }
 
-                Attribute::X => self.x.set(attr.parse(value, LengthDir::Horizontal)?),
-                Attribute::Y => self.y.set(attr.parse(value, LengthDir::Vertical)?),
+                Attribute::X => self.x.set(attr.parse(value)?),
+                Attribute::Y => self.y.set(attr.parse(value)?),
 
                 Attribute::Width => self.w.set(
-                    attr.parse_and_validate(
-                        value,
-                        LengthDir::Horizontal,
-                        Length::check_nonnegative,
-                    )
-                    .map(Some)?,
+                    attr.parse_and_validate(value, LengthHorizontal::check_nonnegative)
+                        .map(Some)?,
                 ),
                 Attribute::Height => self.h.set(
-                    attr.parse_and_validate(value, LengthDir::Vertical, Length::check_nonnegative)
+                    attr.parse_and_validate(value, LengthVertical::check_nonnegative)
                         .map(Some)?,
                 ),
 
@@ -341,12 +330,12 @@ impl NodeTrait for NodeUse {
         let nw = self
             .w
             .get()
-            .unwrap_or_else(|| Length::parse_str("100%", LengthDir::Horizontal).unwrap())
+            .unwrap_or_else(|| LengthHorizontal::parse_str("100%").unwrap())
             .normalize(values, &params);
         let nh = self
             .h
             .get()
-            .unwrap_or_else(|| Length::parse_str("100%", LengthDir::Vertical).unwrap())
+            .unwrap_or_else(|| LengthVertical::parse_str("100%").unwrap())
             .normalize(values, &params);
 
         // width or height set to 0 disables rendering of the element
@@ -422,10 +411,10 @@ impl NodeTrait for NodeSymbol {
         for (attr, value) in pbag.iter() {
             match attr {
                 Attribute::PreserveAspectRatio => {
-                    self.preserve_aspect_ratio.set(attr.parse(value, ())?)
+                    self.preserve_aspect_ratio.set(attr.parse(value)?)
                 }
 
-                Attribute::ViewBox => self.vbox.set(attr.parse(value, ()).map(Some)?),
+                Attribute::ViewBox => self.vbox.set(attr.parse(value).map(Some)?),
 
                 _ => (),
             }
