@@ -4,9 +4,9 @@ use pango_sys;
 use pangocairo;
 use std::cell::{Cell, RefCell};
 
+use allowed_url::Fragment;
 use attributes::Attribute;
 use bbox::BoundingBox;
-use defs::Fragment;
 use drawing_ctx::DrawingCtx;
 use error::{AttributeResultExt, RenderingError};
 use float_eq_cairo::ApproxEqCairo;
@@ -14,9 +14,7 @@ use font_props::FontWeightSpec;
 use length::*;
 use node::{CascadedValues, NodeResult, NodeTrait, NodeType, RsvgNode};
 use parsers::ParseValue;
-use property_bag::PropertyBag;
-use space::{xml_space_normalize, NormalizeDefault, XmlSpaceNormalize};
-use state::{
+use properties::{
     ComputedValues,
     Direction,
     FontStretch,
@@ -29,6 +27,8 @@ use state::{
     XmlLang,
     XmlSpace,
 };
+use property_bag::PropertyBag;
+use space::{xml_space_normalize, NormalizeDefault, XmlSpaceNormalize};
 
 /// An absolutely-positioned array of `Span`s
 ///
@@ -45,15 +45,15 @@ use state::{
 /// [text chunk]: https://www.w3.org/TR/SVG11/text.html#TextLayoutIntroduction
 struct Chunk {
     values: ComputedValues,
-    x: Option<Length>,
-    y: Option<Length>,
+    x: Option<LengthHorizontal>,
+    y: Option<LengthVertical>,
     spans: Vec<Span>,
 }
 
 struct MeasuredChunk {
     values: ComputedValues,
-    x: Option<Length>,
-    y: Option<Length>,
+    x: Option<LengthHorizontal>,
+    y: Option<LengthVertical>,
     advance: (f64, f64),
     spans: Vec<MeasuredSpan>,
 }
@@ -67,29 +67,33 @@ struct PositionedChunk {
 struct Span {
     values: ComputedValues,
     text: String,
-    dx: Option<Length>,
-    dy: Option<Length>,
-    depth: usize,
+    dx: Option<LengthHorizontal>,
+    dy: Option<LengthVertical>,
+    _depth: usize,
 }
 
 struct MeasuredSpan {
     values: ComputedValues,
     layout: pango::Layout,
-    layout_size: (f64, f64),
+    _layout_size: (f64, f64),
     advance: (f64, f64),
-    dx: Option<Length>,
-    dy: Option<Length>,
+    dx: Option<LengthHorizontal>,
+    dy: Option<LengthVertical>,
 }
 
 struct PositionedSpan {
     layout: pango::Layout,
     values: ComputedValues,
-    position: (f64, f64),
+    _position: (f64, f64),
     rendered_position: (f64, f64),
 }
 
 impl Chunk {
-    fn new(values: &ComputedValues, x: Option<Length>, y: Option<Length>) -> Chunk {
+    fn new(
+        values: &ComputedValues,
+        x: Option<LengthHorizontal>,
+        y: Option<LengthVertical>,
+    ) -> Chunk {
         Chunk {
             values: values.clone(),
             x,
@@ -182,8 +186,8 @@ impl Span {
     fn new(
         text: &str,
         values: ComputedValues,
-        dx: Option<Length>,
-        dy: Option<Length>,
+        dx: Option<LengthHorizontal>,
+        dy: Option<LengthVertical>,
         depth: usize,
     ) -> Span {
         Span {
@@ -191,7 +195,7 @@ impl Span {
             text: text.to_string(),
             dx,
             dy,
-            depth,
+            _depth: depth,
         }
     }
 }
@@ -215,7 +219,7 @@ impl MeasuredSpan {
         MeasuredSpan {
             values,
             layout,
-            layout_size: (w, h),
+            _layout_size: (w, h),
             advance,
             dx: span.dx,
             dy: span.dy,
@@ -257,7 +261,7 @@ impl PositionedSpan {
         PositionedSpan {
             layout: measured.layout.clone(),
             values,
-            position: (x, y),
+            _position: (x, y),
             rendered_position: (render_x, render_y),
         }
     }
@@ -415,8 +419,8 @@ fn children_to_chunks(
     node: &RsvgNode,
     cascaded: &CascadedValues<'_>,
     draw_ctx: &mut DrawingCtx,
-    dx: Option<Length>,
-    dy: Option<Length>,
+    dx: Option<LengthHorizontal>,
+    dy: Option<LengthVertical>,
     depth: usize,
 ) {
     for child in node.children() {
@@ -507,8 +511,8 @@ impl NodeChars {
         &self,
         node: &RsvgNode,
         values: &ComputedValues,
-        dx: Option<Length>,
-        dy: Option<Length>,
+        dx: Option<LengthHorizontal>,
+        dy: Option<LengthVertical>,
         depth: usize,
     ) -> Span {
         self.ensure_normalized_string(node, values);
@@ -527,8 +531,8 @@ impl NodeChars {
         node: &RsvgNode,
         values: &ComputedValues,
         chunks: &mut Vec<Chunk>,
-        dx: Option<Length>,
-        dy: Option<Length>,
+        dx: Option<LengthHorizontal>,
+        dy: Option<LengthVertical>,
         depth: usize,
     ) {
         let span = self.make_span(&node, values, dx, dy, depth);
@@ -547,17 +551,17 @@ impl NodeTrait for NodeChars {
 }
 
 pub struct NodeText {
-    x: Cell<Length>,
-    y: Cell<Length>,
-    dx: Cell<Option<Length>>,
-    dy: Cell<Option<Length>>,
+    x: Cell<LengthHorizontal>,
+    y: Cell<LengthVertical>,
+    dx: Cell<Option<LengthHorizontal>>,
+    dy: Cell<Option<LengthVertical>>,
 }
 
 impl NodeText {
     pub fn new() -> NodeText {
         NodeText {
-            x: Cell::new(Length::default()),
-            y: Cell::new(Length::default()),
+            x: Cell::new(Default::default()),
+            y: Cell::new(Default::default()),
             dx: Cell::new(None),
             dy: Cell::new(None),
         }
@@ -587,14 +591,10 @@ impl NodeTrait for NodeText {
     fn set_atts(&self, _: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::X => self.x.set(attr.parse(value, LengthDir::Horizontal)?),
-                Attribute::Y => self.y.set(attr.parse(value, LengthDir::Vertical)?),
-                Attribute::Dx => self
-                    .dx
-                    .set(attr.parse(value, LengthDir::Horizontal).map(Some)?),
-                Attribute::Dy => self
-                    .dy
-                    .set(attr.parse(value, LengthDir::Vertical).map(Some)?),
+                Attribute::X => self.x.set(attr.parse(value)?),
+                Attribute::Y => self.y.set(attr.parse(value)?),
+                Attribute::Dx => self.dx.set(attr.parse(value).map(Some)?),
+                Attribute::Dy => self.dy.set(attr.parse(value).map(Some)?),
                 _ => (),
             }
         }
@@ -624,10 +624,12 @@ impl NodeTrait for NodeText {
 
         let mut positioned_chunks = Vec::new();
         for chunk in &measured_chunks {
-            let normalize = |l: Length| l.normalize(&chunk.values, &params);
-
-            let chunk_x = chunk.x.map_or(x, normalize);
-            let chunk_y = chunk.y.map_or(y, normalize);
+            let chunk_x = chunk
+                .x
+                .map_or_else(|| x, |l| l.normalize(&chunk.values, &params));
+            let chunk_y = chunk
+                .y
+                .map_or_else(|| y, |l| l.normalize(&chunk.values, &params));
 
             let positioned = PositionedChunk::from_measured(&chunk, draw_ctx, chunk_x, chunk_y);
 
@@ -725,10 +727,10 @@ impl NodeTrait for NodeTRef {
 }
 
 pub struct NodeTSpan {
-    x: Cell<Option<Length>>,
-    y: Cell<Option<Length>>,
-    dx: Cell<Option<Length>>,
-    dy: Cell<Option<Length>>,
+    x: Cell<Option<LengthHorizontal>>,
+    y: Cell<Option<LengthVertical>>,
+    dx: Cell<Option<LengthHorizontal>>,
+    dy: Cell<Option<LengthVertical>>,
 }
 
 impl NodeTSpan {
@@ -768,18 +770,10 @@ impl NodeTrait for NodeTSpan {
     fn set_atts(&self, _: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::X => self
-                    .x
-                    .set(attr.parse(value, LengthDir::Horizontal).map(Some)?),
-                Attribute::Y => self
-                    .y
-                    .set(attr.parse(value, LengthDir::Vertical).map(Some)?),
-                Attribute::Dx => self
-                    .dx
-                    .set(attr.parse(value, LengthDir::Horizontal).map(Some)?),
-                Attribute::Dy => self
-                    .dy
-                    .set(attr.parse(value, LengthDir::Vertical).map(Some)?),
+                Attribute::X => self.x.set(attr.parse(value).map(Some)?),
+                Attribute::Y => self.y.set(attr.parse(value).map(Some)?),
+                Attribute::Dx => self.dx.set(attr.parse(value).map(Some)?),
+                Attribute::Dy => self.dy.set(attr.parse(value).map(Some)?),
                 _ => (),
             }
         }
