@@ -10,10 +10,9 @@ use xml_rs::{reader::XmlEvent, ParserConfig};
 use allowed_url::AllowedUrl;
 use attributes::Attribute;
 use create_node::create_node_and_register_id;
-use css::{self, CssStyles};
-use defs::Defs;
+use css::CssStyles;
 use error::LoadingError;
-use handle::{self, RsvgHandle};
+use handle::LoadOptions;
 use io;
 use node::{node_new, Node, NodeType, RsvgNode};
 use property_bag::PropertyBag;
@@ -21,7 +20,6 @@ use structure::NodeSvg;
 use style::NodeStyle;
 use svg::Svg;
 use text::NodeChars;
-use tree::Tree;
 use xml2_load::{xml_state_parse_from_stream, ParseFromStreamError};
 
 #[derive(Clone)]
@@ -70,8 +68,7 @@ extern "C" {
 /// trait objects. Normally the context refers to a `NodeCreationContext` implementation which is
 /// what creates normal graphical elements.
 pub struct XmlState {
-    tree: Option<Tree>,
-    defs: Option<Defs>,
+    tree_root: Option<Rc<Node>>,
     ids: Option<HashMap<String, RsvgNode>>,
     css_styles: Option<CssStyles>,
     context_stack: Vec<Context>,
@@ -79,7 +76,7 @@ pub struct XmlState {
 
     entities: HashMap<String, XmlEntityPtr>,
 
-    handle: *mut RsvgHandle,
+    load_options: LoadOptions,
 }
 
 /// Errors returned from XmlState::acquire()
@@ -95,49 +92,40 @@ enum AcquireError {
 }
 
 impl XmlState {
-    pub fn new(handle: *mut RsvgHandle) -> XmlState {
+    pub fn new(load_options: &LoadOptions) -> XmlState {
         XmlState {
-            tree: None,
-            defs: Some(Defs::new()),
+            tree_root: None,
             ids: Some(HashMap::new()),
             css_styles: Some(CssStyles::new()),
             context_stack: vec![Context::Start],
             current_node: None,
             entities: HashMap::new(),
-            handle,
+            load_options: load_options.clone(),
         }
     }
 
     fn set_root(&mut self, root: &Rc<Node>) {
-        if self.tree.is_some() {
+        if self.tree_root.is_some() {
             panic!("The tree root has already been set");
         }
 
-        self.tree = Some(Tree::new(root));
+        self.tree_root = Some(root.clone());
     }
 
-    pub fn validate_tree(&self) -> Result<(), LoadingError> {
-        if let Some(ref tree) = self.tree {
-            if tree.root_is_svg() {
-                Ok(())
-            } else {
+    pub fn steal_result(&mut self) -> Result<Svg, LoadingError> {
+        match self.tree_root {
+            None => Err(LoadingError::SvgHasNoElements),
+            Some(ref root) if root.get_type() != NodeType::Svg => {
                 Err(LoadingError::RootElementIsNotSvg)
             }
-        } else {
-            Err(LoadingError::SvgHasNoElements)
+            _ => Ok(Svg::new(
+                self.tree_root.take().unwrap(),
+                self.ids.take().unwrap(),
+                self.load_options.clone(),
+            )),
         }
     }
 
-    pub fn steal_result(&mut self) -> Svg {
-        Svg::new(
-            self.handle,
-            self.tree.take().unwrap(),
-            self.defs.take().unwrap(),
-            self.ids.take().unwrap(),
-            self.css_styles.take().unwrap(),
-        )
-    }
-
     fn context(&self) -> Context {
         // We can unwrap since the stack is never empty
         self.context_stack.last().unwrap().clone()
@@ -220,12 +208,12 @@ impl XmlState {
                 && type_.as_ref().map(String::as_str) == Some("text/css")
                 && href.is_some()
             {
-                if let Ok(aurl) = AllowedUrl::from_href(
-                    &href.unwrap(),
-                    handle::get_base_url(self.handle).as_ref(),
-                ) {
+                if let Ok(aurl) =
+                    AllowedUrl::from_href(&href.unwrap(), self.load_options.base_url.as_ref())
+                {
                     // FIXME: handle CSS errors
-                    let _ = handle::load_css(self.css_styles.as_mut().unwrap(), &aurl);
+                    let css_styles = self.css_styles.as_mut().unwrap();
+                    let _ = css_styles.load_css(&aurl);
                 } else {
                     self.error("disallowed URL in xml-stylesheet");
                 }
@@ -287,11 +275,9 @@ impl XmlState {
         if node.get_type() == NodeType::Style {
             let css_data = node.with_impl(|style: &NodeStyle| style.get_css(&node));
 
-            css::parse_into_css_styles(
-                self.css_styles.as_mut().unwrap(),
-                handle::get_base_url(self.handle).clone(),
-                &css_data,
-            );
+            let css_styles = self.css_styles.as_mut().unwrap();
+
+            css_styles.parse(self.load_options.base_url.as_ref(), &css_data);
         }
 
         self.current_node = node.get_parent();
@@ -345,10 +331,6 @@ impl XmlState {
 
         new_node.set_overridden_properties();
 
-        // For now we load resources directly when parsing, but probably we should
-        // move this to a trasversal of the tree once we finished parsing
-        new_node.resolve_resources(&handle::get_load_options(self.handle));
-
         new_node
     }
 
@@ -423,8 +405,8 @@ impl XmlState {
         encoding: Option<&str>,
     ) -> Result<(), AcquireError> {
         if let Some(href) = href {
-            let aurl = AllowedUrl::from_href(href, handle::get_base_url(self.handle).as_ref())
-                .map_err(|e| {
+            let aurl =
+                AllowedUrl::from_href(href, self.load_options.base_url.as_ref()).map_err(|e| {
                     // FIXME: should AlloweUrlError::HrefParseError be a fatal error,
                     // not a resource error?
                     rsvg_log!("could not acquire \"{}\": {}", href, e);
@@ -459,14 +441,14 @@ impl XmlState {
         encoding: Option<&str>,
     ) -> Result<(), AcquireError> {
         let binary = io::acquire_data(aurl, None).map_err(|e| {
-            rsvg_log!("could not acquire \"{}\": {}", aurl.url(), e);
+            rsvg_log!("could not acquire \"{}\": {}", aurl, e);
             AcquireError::ResourceError
         })?;
 
         let encoding = encoding.unwrap_or("utf-8");
 
         let encoder = encoding_from_whatwg_label(encoding).ok_or_else(|| {
-            rsvg_log!("unknown encoding \"{}\" for \"{}\"", encoding, aurl.url());
+            rsvg_log!("unknown encoding \"{}\" for \"{}\"", encoding, aurl);
             AcquireError::FatalError
         })?;
 
@@ -475,7 +457,7 @@ impl XmlState {
             .map_err(|e| {
                 rsvg_log!(
                     "could not convert contents of \"{}\" from character encoding \"{}\": {}",
-                    aurl.url(),
+                    aurl,
                     encoding,
                     e
                 );
@@ -494,10 +476,10 @@ impl XmlState {
             _ => AcquireError::ResourceError,
         })?;
 
-        let load_options = handle::get_load_options(self.handle);
+        let flags = self.load_options.flags;
 
         // FIXME: pass a cancellable
-        xml_state_parse_from_stream(self, &load_options, stream, None).map_err(|e| match e {
+        xml_state_parse_from_stream(self, flags, stream, None).map_err(|e| match e {
             ParseFromStreamError::CouldNotCreateXmlParser => AcquireError::FatalError,
             ParseFromStreamError::IoError(_) => AcquireError::ResourceError,
             ParseFromStreamError::XmlParseError(_) => AcquireError::FatalError,
